program BricxCC;

{$R 'ToolbarBackground.res' 'ToolbarBackground.rc'}

uses
  Forms,
  SysUtils,
  Dialogs,
  Controller in 'Controller.pas' {DirectForm},
  MainUnit in 'MainUnit.pas' {MainForm},
  Diagnose in 'Diagnose.pas' {DiagForm},
  Unlock in 'Unlock.pas' {UnlockForm},
  Watch in 'Watch.pas' {WatchForm},
  About in 'About.pas' {AboutBox},
  GotoLine in 'GotoLine.pas' {GotoForm},
  SearchRCX in 'SearchRCX.pas' {SearchRCXForm},
  Piano in 'Piano.pas' {PianoForm},
  ExecProgram in 'ExecProgram.pas',
  Preferences in 'Preferences.pas' {PrefForm},
  Editor in 'Editor.pas' {EditorForm},
  ConstructUnit in 'ConstructUnit.pas' {ConstructForm},
  JoystickUnit in 'JoystickUnit.pas' {JoystickForm},
  PuzzleUnit in 'PuzzleUnit.pas' {PuzzleForm},
  DatalogUnit in 'DatalogUnit.pas' {DatalogForm},
  MemoryUnit in 'MemoryUnit.pas' {MemoryForm},
  CodeUnit in 'CodeUnit.pas' {CodeForm},
  Translate in 'Translate.pas',
  MessageUnit in 'MessageUnit.pas' {MessageForm},
  DPageSetup in 'DPageSetup.pas' {PageSetupDlg},
  DTestPrintPreview in 'DTestPrintPreview.pas' {TestPrintPreviewDlg},
  ParamUtils in 'ParamUtils.pas',
  GX_IDECodeTemplates in 'GX_IDECodeTemplates.pas',
  RemoteUnit in 'RemoteUnit.pas' {RemoteForm},
  DataAnalysis in 'DataAnalysis.pas' {frmDataAnalysis},
  uChartExport in 'uChartExport.pas' {frmChartExport},
  uChartConfig in 'uChartConfig.pas' {frmChartConfig},
  uSeriesConfig in 'uSeriesConfig.pas' {frmSeriesConfig},
  uPromptSeries in 'uPromptSeries.pas' {frmSeriesPrompt},
  EditCodeTemplate in 'EditCodeTemplate.pas' {frmEditCodeTemplate},
  CodeTemplates in 'CodeTemplates.pas' {frmCodeTemplates},
  uVersionInfo in 'uVersionInfo.pas',
  srecord in 'bricktools\srecord.pas',
  NQCSerial in 'bricktools\NQCSerial.pas',
  rcx_cmd in 'bricktools\rcx_cmd.pas',
  rcx_constants in 'bricktools\rcx_constants.pas',
  rcx_link in 'bricktools\rcx_link.pas',
  RcxLog in 'bricktools\RcxLog.pas',
  scout_def in 'bricktools\scout_def.pas',
  FakeSpirit in 'bricktools\FakeSpirit.pas',
  BricxCC_TLB in 'BricxCC_TLB.pas',
  uBricxCCSpirit in 'uBricxCCSpirit.pas' {BricxCCSpirit: CoClass},
  uBricxCCApp in 'uBricxCCApp.pas' {BricxCCApp: CoClass},
  uCodeExplorer in 'uCodeExplorer.pas' {frmCodeExplorer},
  Transfer in 'Transfer.pas' {frmTransferDlg},
  Transdlg in 'Transdlg.pas' {frmTransEdit},
  uExplorerOptions in 'uExplorerOptions.pas' {frmExplorerOptions},
  uMacroLib in 'uMacroLib.pas',
  uMacroForm in 'uMacroForm.pas' {frmMacroManager},
  uMacroEditor in 'uMacroEditor.pas' {frmMacroEditor},
  uWindowList in 'uWindowList.pas' {frmWindowList},
  uHighlighterProcs in 'uHighlighterProcs.pas';

{$R *.TLB}

{$R *.RES}
{$R 'macro_img.res'}

const
  K_RCXINFO = '(RCX = 0, Cybermaster = 1, Scout = 2, RCX2 = 3)';

resourcestring
  S_RESET = 'reload defaults from Default directory';
  S_COM   = 'use port number N for this instance (5-8 == USB1-4)';
  S_RCX   = 'use brick type N for this instance';
  S_AUTO  = 'auto connect using default or specified port and type';
  S_NONE  = 'do not connect to brick at startup';
  S_POS   = 'position windows using values in INI file';
  S_HELP  = 'display this message before starting up';
  S_NT    = 'use NQC timeout specified for this instance (in seconds)';
  S_DEBUG = 'display debug message when launching NQC';

begin
  Application.Initialize;
  Application.Title := 'Bricx Command Center';

  {Find the program directory}
  ProgramDir := ExtractFilePath(Application.ExeName);

  RegisterApp;

  // we need to check this switch first before we create the preferences form
  if ParamSwitch('/RESET') then
    DeleteMainKey;

  if ParamSwitch('/PTO') then
    PingTimeout := StrToIntDef(ParamValue('/PTO'), K_DEFAULT_PING_TIMEOUT);

  Application.CreateForm(TMainForm, MainForm);
  Application.CreateForm(TDirectForm, DirectForm);
  Application.CreateForm(TDiagForm, DiagForm);
  Application.CreateForm(TWatchForm, WatchForm);
  Application.CreateForm(TSearchRCXForm, SearchRCXForm);
  Application.CreateForm(TPianoForm, PianoForm);
  Application.CreateForm(TConstructForm, ConstructForm);
  Application.CreateForm(TJoystickForm, JoystickForm);
  Application.CreateForm(TDatalogForm, DatalogForm);
  Application.CreateForm(TMemoryForm, MemoryForm);
  Application.CreateForm(TCodeForm, CodeForm);
  Application.CreateForm(TMessageForm, MessageForm);
  Application.CreateForm(TRemoteForm, RemoteForm);
  if not ParamSwitch('/RESET') then
    UpgradeRegistry; // must be done before creating preferences form
  if ParamSwitch('-EMBEDDING') or ParamSwitch('-AUTOMATION') then
  begin
    RunningAsCOMServer := True;
    LocalStartupAction := SU_NOCONNECT;
    Application.ShowMainForm := False;
  end;
  Application.CreateForm(TPrefForm, PrefForm);
  Application.CreateForm(TfrmCodeExplorer, frmCodeExplorer);
  Application.CreateForm(TfrmMacroManager, frmMacroManager);

  if ParamSwitch('/COM') then
    LocalPortNum := StrToIntDef(ParamValue('/COM'), 1);
  if ParamSwitch('/RCX') then
    LocalRCXType := StrToIntDef(ParamValue('/RCX'), SU_RCX);
  if ParamSwitch('/AUTO') then
    LocalStartupAction := SU_CONNECT;
  if ParamSwitch('/NOCONNECT') then
    LocalStartupAction := SU_NOCONNECT;
  // if shell passes the /Print switch (undocumented) then do not connect
  if ParamSwitch('/Print') then
    LocalStartupAction := SU_NOCONNECT;
  if ParamSwitch('/POS') then
    LoadWindowValuesFromFile(ParamValue('/POS'));
  NQCDebug := ParamSwitch('/DEBUG');
  if ParamSwitch('/NT') then
    LocalNQCTimeout := StrToIntDef(ParamValue('/NT'), LocalNQCTimeout) * K_MSTOSEC;
  if ParamSwitch('/HELP') or ParamSwitch('/?') then
  begin
    ShowMessage('/RESET ' + #9#9 + '= ' + S_RESET + #13#10 +
                '/COM=N ' + #9 + '= ' + S_COM + #13#10 +
                '/RCX=N ' + #9 + '= ' + S_RCX + #13#10 +
                ' '       + #9#9 + '   ' + K_RCXINFO + #13#10 +
                '/AUTO '  + #9#9 + '= ' + S_AUTO + #13#10 +
                '/NOCONNECT ' + #9#9 + '= ' + S_NONE + #13#10 +
                '/POS=filename ' + #9 + '= ' + S_POS + #13#10 +
                '/NT=timeout ' + #9 + '= ' + S_NT + #13#10 +
                '/DEBUG ' + #9#9 + '= ' + S_DEBUG + #13#10 +
                '/? or /HELP ' + #9 + '= ' + S_HELP);
  end;
  Application.Run;
end.
