unit Controller;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls, Buttons;

type
  TDirectForm = class(TForm)
    grpMotors: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    grpTasks: TGroupBox;
    TL0: TLabel;
    RunTask0: TSpeedButton;
    StopTask0: TSpeedButton;
    StopTask1: TSpeedButton;
    RunTask1: TSpeedButton;
    TL1: TLabel;
    StopTask2: TSpeedButton;
    RunTask2: TSpeedButton;
    TL2: TLabel;
    StopTask3: TSpeedButton;
    RunTask3: TSpeedButton;
    TL3: TLabel;
    StopTask4: TSpeedButton;
    RunTask4: TSpeedButton;
    TL4: TLabel;
    StopTask5: TSpeedButton;
    RunTask5: TSpeedButton;
    TL5: TLabel;
    StopTask6: TSpeedButton;
    RunTask6: TSpeedButton;
    TL6: TLabel;
    StopTask7: TSpeedButton;
    RunTask7: TSpeedButton;
    TL7: TLabel;
    StopTask8: TSpeedButton;
    RunTask8: TSpeedButton;
    TL8: TLabel;
    StopTask9: TSpeedButton;
    RunTask9: TSpeedButton;
    TL9: TLabel;
    StartAllBtn: TButton;
    StopAllBtn: TButton;
    SensorGroup: TGroupBox;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    SensorType0: TComboBox;
    SensorMode0: TComboBox;
    SensorType1: TComboBox;
    SensorMode1: TComboBox;
    SensorType2: TComboBox;
    SensorMode2: TComboBox;
    grpVariables: TGroupBox;
    VariableBox: TComboBox;
    OperatorBox: TComboBox;
    ValueEdit: TEdit;
    SetBtn: TButton;
    ValueUpDown: TUpDown;
    FwdABtn: TSpeedButton;
    RevABtn: TSpeedButton;
    OffABtn: TSpeedButton;
    FloatABtn: TSpeedButton;
    SpeedA: TTrackBar;
    FwdBBtn: TSpeedButton;
    RevBBtn: TSpeedButton;
    OffBBtn: TSpeedButton;
    FloatBBtn: TSpeedButton;
    SpeedB: TTrackBar;
    FwdCBtn: TSpeedButton;
    RevCBtn: TSpeedButton;
    OffCBtn: TSpeedButton;
    FloatCBtn: TSpeedButton;
    SpeedC: TTrackBar;
    cboSource: TComboBox;
    Label4: TLabel;
    Label5: TLabel;
    procedure SpeedChange(Sender: TObject);
     procedure FormShow(Sender: TObject);
    procedure RunTask0Click(Sender: TObject);
    procedure StopTask0Click(Sender: TObject);
    procedure StartAllBtnClick(Sender: TObject);
    procedure StopAllBtnClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure SensorTypeChange(Sender: TObject);
    procedure SensorModeChange(Sender: TObject);
    procedure SetBtnClick(Sender: TObject);
    procedure FwdABtnClick(Sender: TObject);
    procedure RevABtnClick(Sender: TObject);
    procedure OffABtnClick(Sender: TObject);
    procedure FloatABtnClick(Sender: TObject);
    procedure ValueEditKeyPress(Sender: TObject; var Key: Char);
    procedure ValueEditExit(Sender: TObject);
  private
    { Private declarations }
    procedure InitForm;
  public
    { Public declarations }
  end;

var
  DirectForm: TDirectForm;

implementation

{$R *.DFM}

uses
  SearchRCX, FakeSpirit, rcx_constants;

resourcestring
  sRaw         = 'Raw';
  sBoolean     = 'Boolean';
  sTransCount  = 'Trans Count';
  sPeriodCount = 'Period Count';
  sPercent     = 'Percent';
  sCelsius     = 'Celsius';
  sFahrenheit  = 'Fahrenheit';
  sAngle       = 'Angle';
  sNone        = 'None';
  sSwitch      = 'Switch';
  sTemperature = 'Temperature';
  sLight       = 'Light';
  sScoutSourceError = 'Scout only supports variable and constant sources';

const
  K_HEIGHT = 466;
  K_MOTORS_TOP = 118;
  K_VARS_TOP = 234;
  K_TASKS_TOP = 310;

{Dealing with the Sensors}

procedure TDirectForm.SensorTypeChange(Sender: TObject);
begin
  if Sender = nil then Exit;
  if TComboBox(Sender).ItemIndex >= 0 then
  begin
    FakeRCX.SetSensorType(TComboBox(Sender).Tag, TComboBox(Sender).ItemIndex);
    SensorModeChange(FindComponent('SensorMode' + IntToStr(TComboBox(Sender).Tag)));
  end;
end;

procedure TDirectForm.SensorModeChange(Sender: TObject);
begin
  if Sender = nil then Exit;
  if TComboBox(Sender).ItemIndex >= 0 then
  begin
    FakeRCX.SetSensorMode(TComboBox(Sender).Tag, TComboBox(Sender).ItemIndex, 0);
  end;
end;


{Dealing with the Motors}

function GetMotorNum(tag : integer) : Byte;
begin
  case tag of
    0 : result := 1;
    1 : result := 2;
  else
    result := 4;
  end;
end;

procedure TDirectForm.FwdABtnClick(Sender: TObject);
var
  n : integer;
begin
  if Sender = nil then Exit;
  n := GetMotorNum(TComponent(Sender).Tag);
  FakeRCX.SetFwd(n);
  FakeRCX.MotorsOn(n);
end;

procedure TDirectForm.RevABtnClick(Sender: TObject);
var
  n : integer;
begin
  if Sender = nil then Exit;
  n := GetMotorNum(TComponent(Sender).Tag);
  FakeRCX.SetRwd(n);
  FakeRCX.MotorsOn(n);
end;

procedure TDirectForm.OffABtnClick(Sender: TObject);
begin
  if Sender = nil then Exit;
  FakeRCX.MotorsOff(GetMotorNum(TComponent(Sender).Tag));
end;

procedure TDirectForm.FloatABtnClick(Sender: TObject);
begin
  if Sender = nil then Exit;
  FakeRCX.MotorsFloat(GetMotorNum(TComponent(Sender).Tag));
end;

procedure TDirectForm.SpeedChange(Sender: TObject);
begin
  if Sender = nil then Exit;
  FakeRCX.SetMotorPower(GetMotorNum(TComponent(Sender).Tag), kRCX_ConstantType,
                        TTrackBar(Sender).Position);
end;


{Dealing with the Variables}

procedure TDirectForm.SetBtnClick(Sender: TObject);
var
  vnumb,
  val : integer;
  stype : byte;
begin
  stype := cboSource.ItemIndex;
  if IsScout and not ((stype = 0) or (stype = 2)) then
  begin
    stype := 2;
    cboSource.ItemIndex := stype;
    MessageDlg(sScoutSourceError, mtError, [mbOK], 0);
    Exit;
  end;
  vnumb := VariableBox.ItemIndex;
  val   := ValueUpDown.Position;
  case OperatorBox.ItemIndex of
    0: FakeRCX.SetVar(vnumb,stype,val);
    1: FakeRCX.SumVar(vnumb,stype,val);
    2: FakeRCX.SubVar(vnumb,stype,val);
    3: FakeRCX.MulVar(vnumb,stype,val);
    4: FakeRCX.DivVar(vnumb,stype,val);
    5: FakeRCX.AndVar(vnumb,stype,val);
    6: FakeRCX.OrVar(vnumb,stype,val);
    7: FakeRCX.AbsVar(vnumb,stype,val);
    8: FakeRCX.SgnVar(vnumb,stype,val);
  end;
end;

{Dealing with the Tasks}

procedure TDirectForm.RunTask0Click(Sender: TObject);
begin
  FakeRCX.StartTask(TSpeedButton(Sender).Tag);
end;

procedure TDirectForm.StopTask0Click(Sender: TObject);
begin
  FakeRCX.StopTask(TSpeedButton(Sender).Tag);
end;

procedure TDirectForm.StartAllBtnClick(Sender: TObject);
var
  i:integer;
  c : TCursor;
begin
  if Screen.Cursor = crHourGlass then Exit;
  c := Screen.Cursor;
  try
    Screen.Cursor := crHourGlass;
    if IsRCX then
    begin
      for i:=0 to 9 do FakeRCX.StartTask(i);
    end else if IsScout then
    begin
      for i:=0 to 5 do FakeRCX.StartTask(i);
    end else begin // cybermaster
      for i:=0 to 3 do FakeRCX.StartTask(i);
    end;
  finally
    Screen.Cursor := c;
  end;
end;

procedure TDirectForm.StopAllBtnClick(Sender: TObject);
begin
  FakeRCX.StopAllTasks();
end;

procedure TDirectForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
//  if IsRCX then
//  begin
//    if (Key >='0') and (Key <='9') then
//      FakeRCX.StartTask(Ord(Key)-Ord('0'));
//  end else begin
//    if (Key >='0') and (Key <='3') then
//      FakeRCX.StartTask(Ord(Key)-Ord('0'));
//  end;
end;


{Handling the Window}

procedure TDirectForm.FormShow(Sender: TObject);
var
  c : TCursor;
begin
  c := Screen.Cursor;
  try
    Screen.Cursor := crHourGlass;
    InitForm;
  finally
    Screen.Cursor := c;
  end;
end;

procedure TDirectForm.ValueEditKeyPress(Sender: TObject; var Key: Char);
begin
  if not (Key in  [#8, #16, '-', '0'..'9']) then
    Key := #0;
end;

procedure TDirectForm.ValueEditExit(Sender: TObject);
var
  val : integer;
begin
// ensure value is within range
  val := StrToInt(ValueEdit.Text);
  if val < ValueUpDown.Min then
    ValueEdit.Text := IntToStr(ValueUpDown.Min)
  else if val > ValueUpDown.Max then
    ValueEdit.Text := IntToStr(ValueUpDown.Max);
end;

procedure TDirectForm.InitForm;
var ttt, h, i:integer;
    bVisible : boolean;
    tmpSL : TStrings;
const
  K_FUDGE = 4;
begin
  tmpSL := TStringList.Create;
  try
    {Adapt to RCX type}
    for i:= 4 to 9 do
    begin
      bVisible := IsRCX or (IsScout and (i in [4,5]));
      TSpeedButton(FindComponent('RunTask'+IntToStr(i))).Visible := bVisible;
      TSpeedButton(FindComponent('StopTask'+IntToStr(i))).Visible := bVisible;
      TLabel(FindComponent('TL'+IntToStr(i))).Visible := bVisible;
    end;
    Height              := K_HEIGHT;
    SensorGroup.Visible := True;
    grpMotors.Top       := K_MOTORS_TOP;
    grpVariables.Top    := K_VARS_TOP;
    grpTasks.Top        := K_TASKS_TOP;
    if IsRCX then
    begin
      SensorType0.Enabled := true;
      SensorType1.Enabled := true;
      SensorType2.Enabled := true;
      tmpSL.Clear;
      tmpSL.Append(sRaw);
      tmpSL.Append(sBoolean);
      tmpSL.Append(sTransCount);
      tmpSL.Append(sPeriodCount);
      tmpSL.Append(sPercent);
      tmpSL.Append(sCelsius);
      tmpSL.Append(sFahrenheit);
      tmpSL.Append(sAngle);
      SensorMode0.Items.Assign(tmpSL);
      SensorMode1.Items.Assign(tmpSL);
      SensorMode2.Items.Assign(tmpSL);
      tmpSL.Clear;
      tmpSL.Append(sNone);
      tmpSL.Append(sSwitch);
      tmpSL.Append(sTemperature);
      tmpSL.Append(sLight);
      tmpSL.Append(sAngle);
      SensorType0.Items.Assign(tmpSL);
      SensorType1.Items.Assign(tmpSL);
      SensorType2.Items.Assign(tmpSL);
    end else if IsScout then begin
      // scout
      SensorGroup.Visible := False;
      grpMotors.Top       := K_MOTORS_TOP - SensorGroup.Height - K_FUDGE;
      grpVariables.Top    := K_VARS_TOP - SensorGroup.Height - K_FUDGE;
      grpTasks.Top        := K_TASKS_TOP - SensorGroup.Height - K_FUDGE;
      Height              := K_HEIGHT  - SensorGroup.Height - K_FUDGE*2;
    end else begin
      // cybermaster
      SensorGroup.Visible := True;
      SensorType0.Enabled := false;
      SensorType1.Enabled := false;
      SensorType2.Enabled := false;
      tmpSL.Clear;
      tmpSL.Append(sRaw);
      tmpSL.Append(sBoolean);
      tmpSL.Append(sTransCount);
      tmpSL.Append(sPeriodCount);
      tmpSL.Append(sPercent);
      SensorMode0.Items.Assign(tmpSL);
      SensorMode1.Items.Assign(tmpSL);
      SensorMode2.Items.Assign(tmpSL);
      tmpSL.Clear;
  {
      tmpSL.Append('None');
      tmpSL.Append('Switch');
      tmpSL.Append('Temperature');
      tmpSL.Append('Light');
      tmpSL.Append('Angle');
  }
      SensorType0.Items.Assign(tmpSL);
      SensorType1.Items.Assign(tmpSL);
      SensorType2.Items.Assign(tmpSL);
    end;
  finally
    tmpSL.Free;
  end;

  // variables
  h := 31;
  if IsScout then
    h := 9;
  VariableBox.Items.Clear;
  for i := 0 to h do
    VariableBox.Items.Add(Format('Var %2d',[i]));
  VariableBox.ItemIndex := 0;
  OperatorBox.ItemIndex := 0;
  cboSource.ItemIndex   := 2;

  try
    {Set the motor items}
    ttt := FakeRCX.GetOutputStatus(0);
    FwdABtn.Down    :=  (ttt div 128 = 1) and ((ttt mod 16) div 8 = 1);
    RevABtn.Down    :=  (ttt div 128 = 1) and ((ttt mod 16) div 8 = 0);
    OffABtn.Down    :=  (ttt div 128 = 0) and ((ttt mod 128) div 64 = 1);
    FloatABtn.Down  :=  (ttt div 128 = 0) and ((ttt mod 128) div 64 = 0);
    SpeedA.Position := ttt mod 8;

    ttt := FakeRCX.GetOutputStatus(1);
    FwdBBtn.Down    :=  (ttt div 128 = 1) and ((ttt mod 16) div 8 = 1);
    RevBBtn.Down    :=  (ttt div 128 = 1) and ((ttt mod 16) div 8 = 0);
    OffBBtn.Down    :=  (ttt div 128 = 0) and ((ttt mod 128) div 64 = 1);
    FloatBBtn.Down  :=  (ttt div 128 = 0) and ((ttt mod 128) div 64 = 0);
    SpeedB.Position := ttt mod 8;

    ttt := FakeRCX.GetOutputStatus(2);
    FwdCBtn.Down    :=  (ttt div 128 = 1) and ((ttt mod 16) div 8 = 1);
    RevCBtn.Down    :=  (ttt div 128 = 1) and ((ttt mod 16) div 8 = 0);
    OffCBtn.Down    :=  (ttt div 128 = 0) and ((ttt mod 128) div 64 = 1);
    FloatCBtn.Down  :=  (ttt div 128 = 0) and ((ttt mod 128) div 64 = 0);
    SpeedC.Position := ttt mod 8;

    if not IsScout then
    begin
      { Set the sensor Items }
      SensorType0.ItemIndex := FakeRCX.Poll(kRCX_InputTypeType,0);
      SensorType1.ItemIndex := FakeRCX.Poll(kRCX_InputTypeType,1);
      SensorType2.ItemIndex := FakeRCX.Poll(kRCX_InputTypeType,2);

      SensorMode0.ItemIndex := FakeRCX.Poll(kRCX_InputModeType,0) div 32;
      SensorMode1.ItemIndex := FakeRCX.Poll(kRCX_InputModeType,1) div 32;
      SensorMode2.ItemIndex := FakeRCX.Poll(kRCX_InputModeType,2) div 32;
    end
    else
    begin
    end;
  except
    ;
  end;
end;

end.
