unit DataAnalysis;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, TeeProcs, TeEngine, Chart, Series, StdCtrls, Menus, ActnList,
  ImgList, ToolWin, ComCtrls, Buttons;

type
  TfrmDataAnalysis = class(TForm)
    chtData: TChart;
    ActionList1: TActionList;
    ImageList1: TImageList;
    actPrint: TAction;
    actConfig: TAction;
    actExport: TAction;
    actCopy: TAction;
    actClose: TAction;
    dlgPrint: TPrintDialog;
    Panel1: TPanel;
    Panel2: TPanel;
    pnlVCR: TPanel;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    Bevel1: TBevel;
    btnFirst: TSpeedButton;
    btnPrev: TSpeedButton;
    btnNext: TSpeedButton;
    btnLast: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure actPrintExecute(Sender: TObject);
    procedure actConfigExecute(Sender: TObject);
    procedure actCopyExecute(Sender: TObject);
    procedure actExportExecute(Sender: TObject);
    procedure actCloseExecute(Sender: TObject);
    procedure chtDataMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnFirstClick(Sender: TObject);
    procedure btnPrevClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnLastClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    fData: TStrings;
    fDataIsXY: boolean;
    FChartType: integer;
    fFromWatch: boolean;
    fWatchPoints: byte;
    procedure SetData(const Value: TStrings);
    procedure PrepareChartXY;
    procedure PrepareChartNormal;
    procedure SetChartType(const Value: integer);
    procedure UpdateVCRButtons;
    function  PromptForSeries : TChartSeries;
  public
    { Public declarations }
    procedure AddNewData(aValues : TStrings);
    procedure PrepareChart;
    property  Data : TStrings read fData write SetData;
    property  DataIsXY : boolean read fDataIsXY write fDataIsXY;
    property  UseChartType : integer read FChartType write SetChartType;
    property  FromWatch : boolean read fFromWatch write fFromWatch;
    property  WatchPoints : byte read fWatchPoints write fWatchPoints;
  end;

implementation

{$R *.DFM}

uses
  Printers, Clipbrd, uChartExport, uChartConfig, uSeriesConfig,
  uPromptSeries, Watch;

{ TfrmDataAnalysis }

procedure TfrmDataAnalysis.SetData(const Value: TStrings);
begin
  fData.Clear;
  fData.Assign(Value);
  PrepareChart;
end;

procedure TfrmDataAnalysis.FormCreate(Sender: TObject);
begin
  fWatchPoints := 10;
  fFromWatch   := False;
  fChartType   := 0;
  fDataIsXY    := False;
  fData        := TStringList.Create;
end;

procedure TfrmDataAnalysis.FormDestroy(Sender: TObject);
begin
  if fFromWatch then
  begin
    if Assigned(WatchForm) then
      WatchForm.GraphDestroyed;
  end;
  fData.Free;
end;

procedure TfrmDataAnalysis.PrepareChart;
begin
  // delete any existing series first
  while chtData.SeriesCount > 0 do
    chtData.Series[0].Free;

  // now load new series
  if fDataIsXY then
    PrepareChartXY
  else
    PrepareChartNormal;
end;

procedure TfrmDataAnalysis.actPrintExecute(Sender: TObject);
begin
  if dlgPrint.Execute then
  begin
    if Printer.Orientation = poLandscape then
      chtData.PrintLandscape
    else
      chtData.Print;
  end;
end;

procedure TfrmDataAnalysis.actConfigExecute(Sender: TObject);
begin
  with TfrmChartConfig.Create(nil) do
  try
    ChartType := UseChartType;
    TheChart := chtData;
    if ShowModal = mrOK then
    begin
      CopySettings;
      UseChartType := ChartType;
      chtData.Refresh;
      UpdateVCRButtons;
      pnlVCR.Visible := chtData.MaxPointsPerPage > 0;
    end;
  finally
    Free;
  end;
end;

procedure TfrmDataAnalysis.actCopyExecute(Sender: TObject);
var
  i, j : integer;
  tmpStr : string;
begin
  if chtData.SeriesCount = 0 then Exit;
  // copy to clipboard
  // if we are running from the Watch window we should copy all the data and
  // not just the last N points so to make this simple we reload that data
  if FromWatch then
    PrepareChart;
    
  tmpStr := '';
  for j := 0 to chtData.SeriesCount - 1 do
  begin
    tmpStr := tmpStr + chtData.Series[j].Title;
    if j < chtData.SeriesCount - 1 then
      tmpStr := tmpStr + #9;
  end;
  tmpStr := tmpStr + #13#10;
  for i := 0 to chtData.Series[0].YValues.Count - 1 do
  begin
    for j := 0 to chtData.SeriesCount - 1 do
    begin
      tmpStr := tmpStr + FloatToStr(chtData.Series[j].YValues[i]);
      if j < chtData.SeriesCount - 1 then
        tmpStr := tmpStr + #9;
    end;
    tmpStr := tmpStr + #13#10;
  end;
  ClipBoard.AsText := tmpStr;
end;

procedure TfrmDataAnalysis.actExportExecute(Sender: TObject);
begin
  TfrmChartExport.DoExport(chtData);
end;

procedure TfrmDataAnalysis.actCloseExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmDataAnalysis.PrepareChartNormal;
var
  i, j, cPos : integer;
  tmpStr, sType : string;
  s, sTmp : TChartSeries;
  val : integer;
begin
  for i := 0 to fData.Count - 1 do
  begin
    tmpStr := fData[i];
    cPos   := Pos(':', tmpStr);
    // the type is the everything up to the colon
    sType := Copy(tmpStr, 1, cPos-1);
    Delete(tmpStr, 1, cPos);
    // the rest is the value
    val := StrToIntDef(tmpStr, 0);
    s := nil;
    for j := 0 to chtData.SeriesCount - 1 do
    begin
      sTmp := chtData.Series[j];
      if sTmp.Title = sType then
      begin
        s := sTmp;
        Break;
      end;
    end;
    if not Assigned(s) then
    begin
      // create a new series
      case UseChartType of
        0 : s := TLineSeries.Create( Self );
        1 : s := TBarSeries.Create( Self );
        2 : s := THorizBarSeries.Create( Self );
        3 : s := TAreaSeries.Create( Self );
      else
        s := TPointSeries.Create( Self );
      end;
//      s := TLineSeries.Create( Self );
      s.ParentChart   := chtData;
      s.Title         := sType;
      s.XValues.Order := loNone;
      s.Marks.Visible := False;
    end;
    // now add a data point
    s.AddXY(s.Count, val, '', clTeeColor);
//    s.Add(val, '', clTeeColor);
  end;
end;

procedure TfrmDataAnalysis.PrepareChartXY;
var
  i, cPos : integer;
  tmpStr : string;
  s : TChartSeries;
  xval, yval : integer;
begin
  // only one series
  case UseChartType of
    0 : s := TLineSeries.Create( Self );
    1 : s := TBarSeries.Create( Self );
    2 : s := THorizBarSeries.Create( Self );
    3 : s := TAreaSeries.Create( Self );
  else
    s := TPointSeries.Create( Self );
  end;
  s.ParentChart   := chtData;
  s.Title         := 'Datalog';
  s.Marks.Visible := False;
//  s.XValues.Order := loNone;
  i := 0;
  while i < fData.Count do
  begin
    tmpStr := fData[i];
    cPos   := Pos(':', tmpStr);
    Delete(tmpStr, 1, cPos);
    // the rest is the value
    xval := StrToIntDef(tmpStr, 0);
    tmpStr := fData[i+1];
    cPos   := Pos(':', tmpStr);
    Delete(tmpStr, 1, cPos);
    // the rest is the value
    yval := StrToIntDef(tmpStr, 0);
    // now add a data point
    s.AddXY(xval, yval, '', clTeeColor);
    inc(i, 2);
  end;
end;

procedure TfrmDataAnalysis.SetChartType(const Value: integer);
begin
  if FChartType <> Value then
  begin
    FChartType := Value;
    PrepareChart;
  end;
end;

procedure TfrmDataAnalysis.chtDataMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  p : TChartClickedPart;
  S : TChartSeries;
begin
  if (ssCtrl in Shift) and (Button = mbLeft) then
  begin
    S := nil;
    chtData.CalcClickedPart(Point(X, Y), p);
    if p.Part = cpSeries then
    begin
      S := p.ASeries;
    end
    else if p.Part = cpChartRect then
    begin
      // prompt for series
      S := PromptForSeries;
    end;
    if Assigned(S) then
    begin
      with TfrmSeriesConfig.Create(nil) do
      try
        ChartType := UseChartType;
        TheSeries := S;
        if ShowModal = mrOK then
        begin
          CopySeriesSettings;
          chtData.Refresh;
        end;
      finally
        Free;
      end;
    end;
  end;
end;

procedure TfrmDataAnalysis.btnFirstClick(Sender: TObject);
begin
  chtData.UndoZoom;
  chtData.Page := 1;
  UpdateVCRButtons;
end;

procedure TfrmDataAnalysis.btnPrevClick(Sender: TObject);
begin
  chtData.UndoZoom;
  chtData.PreviousPage;
  UpdateVCRButtons;
end;

procedure TfrmDataAnalysis.btnNextClick(Sender: TObject);
begin
  chtData.UndoZoom;
  chtData.NextPage;
  UpdateVCRButtons;
end;

procedure TfrmDataAnalysis.btnLastClick(Sender: TObject);
begin
  chtData.UndoZoom;
  chtData.Page := chtData.NumPages;
  UpdateVCRButtons;
end;

procedure TfrmDataAnalysis.UpdateVCRButtons;
var
  p, n : integer;
begin
  p := chtData.Page;
  n := chtData.NumPages;
  btnFirst.Enabled := p > 1;
  btnPrev.Enabled  := p > 1;
  btnNext.Enabled  := p < n;
  btnLast.Enabled  := p < n;
end;

procedure TfrmDataAnalysis.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TfrmDataAnalysis.AddNewData(aValues: TStrings);
var
  i, j, cPos : integer;
  tmpStr, sType : string;
  s, sTmp : TChartSeries;
  val : integer;
begin
  // create series if needed and add data points
  for i := 0 to aValues.Count - 1 do
  begin
    tmpStr := aValues[i];
    cPos   := Pos(':', tmpStr);
    // the type is the everything up to the colon
    sType := Copy(tmpStr, 1, cPos-1);
    Delete(tmpStr, 1, cPos);
    // the rest is the value
    val := StrToIntDef(tmpStr, 0);
    s := nil;
    for j := 0 to chtData.SeriesCount - 1 do
    begin
      sTmp := chtData.Series[j];
      if sTmp.Title = sType then
      begin
        s := sTmp;
        Break;
      end;
    end;
    if not Assigned(s) then
    begin
      // create a new series
      case UseChartType of
        0 : s := TLineSeries.Create( Self );
        1 : s := TBarSeries.Create( Self );
        2 : s := THorizBarSeries.Create( Self );
        3 : s := TAreaSeries.Create( Self );
      else
        s := TPointSeries.Create( Self );
      end;
      s.ParentChart   := chtData;
      s.Title         := sType;
      s.XValues.Order := loNone;
      s.Marks.Visible := False;
    end;
    // now add a data point
    if s.Count > WatchPoints then
      s.Delete(0);
    if s.Count = 0 then
      s.AddXY(s.Count, val, '', clTeeColor)
    else
      s.AddXY(Trunc(s.XValues.Last) + 1, val, '', clTeeColor);
//    s.AddXY(s.Count, val, '', clTeeColor);
  end;
  // add data in aValues to fData
  fData.AddStrings(aValues);
end;

function TfrmDataAnalysis.PromptForSeries: TChartSeries;
var
  i : integer;
begin
  result := nil;
  i := TfrmSeriesPrompt.PromptForSeries(chtData);
  if i <> -1 then
    result := chtData.Series[i];
end;

end.
