unit DatalogUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TDatalogForm = class(TForm)
    DatalogMemo: TMemo;
    UploadBtn: TButton;
    ClearBtn: TButton;
    SizeBox: TComboBox;
    Label1: TLabel;
    SaveBtn: TButton;
    SaveDialog1: TSaveDialog;
    btnAnalyze: TButton;
    btnAnalyzeXY: TButton;
    procedure UploadBtnClick(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure SizeBoxChange(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure btnAnalyzeClick(Sender: TObject);
    procedure btnAnalyzeXYClick(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateButtonState;
    procedure LaunchAnalysis(bXY : boolean = false);
  public
    { Public declarations }
  end;

var
  DatalogForm: TDatalogForm;

implementation

{$R *.DFM}

uses
  FakeSpirit, DataAnalysis;

procedure TDatalogForm.UploadBtnClick(Sender: TObject);
var
  d : TStrings;
  c : TCursor;
begin
  DatalogMemo.Lines.Clear;
  c := Screen.Cursor;
  try
    Screen.Cursor := crHourGlass;
    d := FakeRCX.UploadDatalog(True);
    DatalogMemo.Lines.Assign(d);
    SizeBox.Text := IntToStr(d.Count);
    UpdateButtonState;
  finally
    Screen.Cursor := c;
  end;
end;

procedure TDatalogForm.ClearBtnClick(Sender: TObject);
begin
  FakeRCX.SetDatalog(0);
  SizeBox.Text := '0';
end;

procedure TDatalogForm.SizeBoxChange(Sender: TObject);
begin
  FakeRCX.SetDatalog(StrToIntDef(SizeBox.Text, 0));
end;

procedure TDatalogForm.SaveBtnClick(Sender: TObject);
begin
  if SaveDialog1.Execute() then
  begin
    DatalogMemo.Lines.SaveToFile(SaveDialog1.FileName);
  end;
end;

procedure TDatalogForm.btnAnalyzeClick(Sender: TObject);
begin
  LaunchAnalysis;
end;

procedure TDatalogForm.UpdateButtonState;
var
  bEnable : boolean;
begin
  bEnable := DatalogMemo.Lines.Count > 0;
  btnAnalyze.Enabled   := bEnable;
  btnAnalyzeXY.Enabled := bEnable;
end;

procedure TDatalogForm.LaunchAnalysis(bXY: boolean);
var
  F : TfrmDataAnalysis;
begin
  // create and show analysis form here
  F := TfrmDataAnalysis.Create(Application);
  F.DataIsXY := bXY;
  F.Data     := DatalogMemo.Lines;
  F.Show;
end;

procedure TDatalogForm.btnAnalyzeXYClick(Sender: TObject);
begin
  LaunchAnalysis(True);
end;

end.
