unit Diagnose;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Mask, Spin;

type
  TDiagForm = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Alive: TPanel;
    Label2: TLabel;
    Version: TPanel;
    ttt: TLabel;
    RefreshBtn: TButton;
    Label3: TLabel;
    Port: TPanel;
    IRGroup: TGroupBox;
    IRShort: TRadioButton;
    IRLong: TRadioButton;
    Label4: TLabel;
    ProgramNumb: TPanel;
    WatchGroup: TGroupBox;
    Label6: TLabel;
    CurrentBtn: TButton;
    Watch: TPanel;
    DisplayGroup: TGroupBox;
    DisplayType: TComboBox;
    PowerGroup: TGroupBox;
    Label5: TLabel;
    BatStatus: TPanel;
    rbVar: TRadioButton;
    rbConst: TRadioButton;
    PowerDown: TSpinEdit;
    procedure FormShow(Sender: TObject);
    procedure RefreshBtnClick(Sender: TObject);
    procedure IRShortClick(Sender: TObject);
    procedure IRLongClick(Sender: TObject);
    procedure CurrentBtnClick(Sender: TObject);
    procedure DisplayTypeChange(Sender: TObject);
    procedure PowerDownChange(Sender: TObject);
    procedure rbVarClick(Sender: TObject);
  private
    { Private declarations }
    procedure InitForm;
    procedure AdjustFormSize;
  public
    { Public declarations }
  end;

var
  DiagForm: TDiagForm;

implementation

{$R *.DFM}
uses
  Preferences, SearchRCX, FakeSpirit, rcx_constants;

resourcestring
  sRCXDead = 'Brick is NOT alive';
  sRCXAlive = 'Brick is alive';
  sVariable = 'Variable ';

const
  carray : array[0..7] of string = (
    'Watch', 'Input 1', 'Input 2', 'Input 3', 'Output A', 'Output B',
    'Output C', 'User Select');
  K_DEFAULT_WATCH = '00:00';
  K_DEFAULT_CAPTION = '---';

procedure TDiagForm.FormShow(Sender: TObject);
var
  c : TCursor;
begin
  c := Screen.Cursor;
  try
    Screen.Cursor := crHourGlass;
    InitForm;
  finally
    Screen.Cursor := c;
  end;
end;

procedure TDiagForm.RefreshBtnClick(Sender: TObject);
var
  c : TCursor;
  w : integer;
  rom, ram : Cardinal;
begin
  c := Screen.Cursor;
  try
    Screen.Cursor := crHourGlass;
    Port.Caption  := FakeRCX.NicePortName;
    if FakeRCX.BrickAlive then
    begin
      Alive.Caption := sRCXAlive;
      try
        BatStatus.Caption := Format('%d mV',[FakeRCX.BatteryLevel]);
      except
        BatStatus.Caption := K_DEFAULT_CAPTION;
      end;
      FakeRCX.Version(rom, ram);
      Version.Caption := Format('%8.8x/%8.8x', [rom, ram]);
      if IsRCX then
      begin
        try
          ProgramNumb.Caption := Format('%d',[1+FakeRCX.Poll(kRCX_ProgramSlotType,0)]);
        except
          ProgramNumb.Caption := K_DEFAULT_CAPTION;
        end;
        try
          w := FakeRCX.Poll(kRCX_WatchType,0);
          Watch.Caption := Format('%d:%d',[w div 60,w mod 60]);
        except
          Watch.Caption := K_DEFAULT_WATCH;
        end;
      end else begin
        ProgramNumb.Caption := K_DEFAULT_CAPTION;
        Watch.Caption := K_DEFAULT_CAPTION;
      end;
    end else begin
      Alive.Caption       := sRCXDead;
      BatStatus.Caption   := K_DEFAULT_CAPTION;
      Version.Caption     := K_DEFAULT_CAPTION;
      ProgramNumb.Caption := K_DEFAULT_CAPTION;
      Watch.Caption       := K_DEFAULT_WATCH;
    end;
  finally
    Screen.Cursor := c;
  end;
end;

procedure TDiagForm.IRShortClick(Sender: TObject);
begin
  FakeRCX.TransmitPower(tlNear);
end;

procedure TDiagForm.IRLongClick(Sender: TObject);
begin
  FakeRCX.TransmitPower(tlFar);
end;

procedure TDiagForm.CurrentBtnClick(Sender: TObject);
var
  Hour, Min, Sec, MSec: Word;
begin
  DecodeTime(Now, Hour, Min, Sec, MSec);
  Watch.Caption:= Format('%d:%d',[Hour,Min]);
  FakeRCX.SetWatch(Hour, Min);
end;

procedure TDiagForm.DisplayTypeChange(Sender: TObject);
begin
  if rbVar.Checked then
    FakeRCX.SelectDisplay(kRCX_VariableType, DisplayType.ItemIndex)
  else
    FakeRCX.SelectDisplay(kRCX_ConstantType, DisplayType.ItemIndex);
end;

procedure TDiagForm.PowerDownChange(Sender: TObject);
begin
  FakeRCX.PowerDownTime(PowerDown.Value);
end;

procedure TDiagForm.rbVarClick(Sender: TObject);
var
  i : integer;
  d : integer;
begin
  DisplayType.Items.Clear;
  if Sender = rbVar then
  begin
    for i := 0 to 31 do
    begin
      DisplayType.Items.Add(sVariable + IntToStr(i));
    end;
  end
  else
  begin
    d := 1;
    if LocalRCXType = SU_RCX2 then
      d := 0;
    for i := low(carray) to high(carray) - d do
    begin
      DisplayType.Items.Add(carray[i]);
    end;
  end;
end;

procedure TDiagForm.InitForm;
begin
  rbVarClick(rbConst);
  DisplayGroup.Visible := IsRCX;
  WatchGroup.Visible := IsRCX;
  IRGroup.Visible := IsRCX or IsScout;
  AdjustFormSize;
  RefreshBtnClick(Self);
  if IsRCX or IsScout then
    if IRShort.Checked then
    begin
      FakeRCX.TransmitPower(tlNear);
    end
    else
    begin
      FakeRCX.TransmitPower(tlFar);
    end;
end;

procedure TDiagForm.AdjustFormSize;
const
  K_FUDGE = 2;
  K_HEIGHT = 430;
begin
  if IsRCX then
  begin
    Height := K_HEIGHT;
  end
  else if IsScout then
  begin
    Height := WatchGroup.Top + GetSystemMetrics(SM_CYCAPTION) + K_FUDGE;
  end
  else
  begin
    Height := IRGroup.Top + GetSystemMetrics(SM_CYCAPTION) + K_FUDGE;
  end;
end;

end.
