program FakeSpiritTest;

uses
  Classes,
  ParamUtils,
  SysUtils,
  Dialogs,
  FakeSpirit in 'bricktools\FakeSpirit.pas';

{$R *.RES}

var
  remoteRepeat : integer;

procedure PrintUsage;
var
  progName : string;
begin
  progName := ExtractFileName(ParamStr(0));
  progName := Copy(progName, 1, Length(progName)-4);
	WriteLn(progName + ' - version 0.1b');
	WriteLn('Usage: ' + progName + ' [options] [actions]');
	WriteLn('Options:');
	WriteLn('   /RCX=n: target can be RCX (0), Cybermaster (1), Scout (2), or RCX2 (3)');
	WriteLn('   /COM=n: specify port (1..8 : 5-8 == USB1-4)');
	WriteLn('   /RPT=n: specify remote repeat count');
	WriteLn('   /T=n: specify transmit timeout value');
	WriteLn('   /V: specify verbose mode');
	WriteLn('   /F: specify fast mode');
	WriteLn('Actions:');
	WriteLn('   -run: run current program');
	WriteLn('   -pgm=n: select program number');
	WriteLn('   -datalog | -datalog_full: upload datalog (_full == verbose)');
	WriteLn('   -near : set IR to near mode');
	WriteLn('   -far : set IR to far mode');
  WriteLn('   -firmware=filename : download firmware');
  WriteLn('   -firmfast=filename : download firmware at quad speed');
	WriteLn('   -watch=<HHMM|now> : set RCX time');
	WriteLn('   -sleep=<timeout> : set RCX sleep timeout');
	WriteLn('   -msg=<number> : send IR message to RCX');
	WriteLn('   -raw=<data> : format data as a packet and send to RCX');
	WriteLn('   -remote=<value> : send a remote command to RCX');
	WriteLn('   -scout=<0|1> : put Scout into normal or power mode');
	WriteLn('   -clear : erase all programs and datalog in RCX');
	WriteLn('   -help : display command line options');
end;

begin

  remoteRepeat := 1;

  if ParamSwitch('-help') then
  begin
    PrintUsage;
    Exit;
  end;

  if ParamSwitch('/RPT') then
    remoteRepeat := StrToIntDef(ParamValue('/RPT'), 1);
  if ParamSwitch('/COM') then
    FakeRCX.PortNum := StrToIntDef(ParamValue('/COM'), 1);
  if ParamSwitch('/RCX') then
    FakeRCX.RCXType := StrToIntDef(ParamValue('/RCX'), 0);
  FakeRCX.VerboseMode := ParamSwitch('/V');
  FakeRCX.FastMode := ParamSwitch('/F');
  if ParamSwitch('/T') then
    FakeRCX.RxTimeout := StrToIntDef(ParamValue('/T'), 1000);
  if ParamSwitch('-scout') then
    FakeRCX.Scout(StrToIntDef(ParamValue('-scout'), 1));
  if ParamSwitch('-datalog') then
    FakeRCX.UploadDatalog(false);
  if ParamSwitch('-datalog_full') then
    FakeRCX.UploadDatalog(true);
  if ParamSwitch('-clear') then
    FakeRCX.ClearMemory;
  if ParamSwitch('-near') then
    FakeRCX.TransmitPower(tlNear);
  if ParamSwitch('-far') then
    FakeRCX.TransmitPower(tlFar);
  if ParamSwitch('-firmware') then
    FakeRCX.DownloadFirmware(ParamValue('-firmware'), false);
  if ParamSwitch('-firmfast') then
    FakeRCX.DownloadFirmware(ParamValue('-firmfast'), true);
  if ParamSwitch('-watch') then
    FakeRCX.SetWatch(ParamValue('-watch'));
  if ParamSwitch('-sleep') then
    FakeRCX.Sleep(StrToIntDef(ParamValue('-sleep'), 10));
  if ParamSwitch('-run') then
    FakeRCX.StartTask(StrToIntDef(ParamValue('-run'), 0));
  if ParamSwitch('-pgm') then
    FakeRCX.SelectProgram(StrToIntDef(ParamValue('-pgm'), 1));
  if ParamSwitch('-msg') then
    FakeRCX.SendMessage(StrToIntDef(ParamValue('-msg'), 0));
  if ParamSwitch('-raw') then
    FakeRCX.SendRawCommand(ParamValue('-raw'), true);
  if ParamSwitch('-raw1') then
    FakeRCX.SendRawCommand(ParamValue('-raw1'), false);
  if ParamSwitch('-remote') then
    FakeRCX.SendRemote(ParamValue('-remote'), remoteRepeat);

  if FakeRCX.DataLog.Count > 0 then
  begin
    ShowMessage('datalog');
  end;

end.
