unit JoystickUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, Buttons;

type
  TJoystickForm = class(TForm)
    JoyTimer: TTimer;
    LeftGroup: TGroupBox;
    LMotorA: TRadioButton;
    LMotorB: TRadioButton;
    LMotorC: TRadioButton;
    LReversed: TCheckBox;
    RightGroup: TGroupBox;
    RMotorA: TRadioButton;
    RMotorB: TRadioButton;
    RMotorC: TRadioButton;
    RReversed: TCheckBox;
    GroupBox3: TGroupBox;
    SpeedBar: TTrackBar;
    ModeGroup: TGroupBox;
    LeftRightBtn: TRadioButton;
    DriveSteerBtn: TRadioButton;
    MoveGroup: TGroupBox;
    DirBtn7: TSpeedButton;
    DirBtn8: TSpeedButton;
    DirBtn9: TSpeedButton;
    DirBtn4: TSpeedButton;
    DirBtn5: TSpeedButton;
    DirBtn6: TSpeedButton;
    DirBtn1: TSpeedButton;
    DirBtn2: TSpeedButton;
    DirBtn3: TSpeedButton;
    Task1Btn: TBitBtn;
    Task2Btn: TBitBtn;
    procedure JoyTimerTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure LReversedClick(Sender: TObject);
    procedure RReversedClick(Sender: TObject);
    procedure SpeedBarChange(Sender: TObject);
    procedure RMotorClick(Sender: TObject);
    procedure LMotorClick(Sender: TObject);
    procedure LeftRightBtnClick(Sender: TObject);
    procedure DriveSteerBtnClick(Sender: TObject);
    procedure DirBtnMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure DirBtnClick(Sender: TObject);
    procedure TaskBtnClick(Sender: TObject);
    procedure DirBtnMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  JoystickForm: TJoystickForm;

implementation

{$R *.DFM}

uses MMSystem, Preferences, FakeSpirit;

var oldldir:integer =100;                   // previous left motor direction
    oldrdir:integer =100;                   // previous right motor direction
    oldspeed:integer = 100;                 // previous motor speed
    oldbuttons:array[1..4] of boolean=      // previous status of buttons
               (false,false,false,false) ;

    ldirA:array[1..9] of integer =
               ( 0,-1,-1,-1, 0, 1, 0, 1, 1);
    rdirA:array[1..9] of integer =
               (-1,-1, 0, 1, 0,-1, 1, 1, 0);
    ldirB:array[1..9] of integer =
               (-1,-1,-1, 0, 0, 0, 1, 1, 1);
    rdirB:array[1..9] of integer =
               (-1, 0, 1,-1, 0, 1,-1, 0, 1);


function GetMotorList(m1 : integer; m2 : integer = -1) : Byte;
var
  b1, b2 : Byte;
begin
  case m1 of
     0 : b1 := 1;
     1 : b1 := 2;
     2 : b1 := 4;
  else
    b1 := 0;
  end;
  case m2 of
     0 : b2 := 1;
     1 : b2 := 2;
     2 : b2 := 4;
  else
    b2 := 0;
  end;
  result := b1 + b2;
end;

procedure MoveRCX(dir:integer);
{Computes the correct directions and set the motors}
var
  ldir,rdir:integer;
  l, r, lr : Byte;
begin
  {Show direction}
  TSpeedButton(JoystickForm.FindComponent('DirBtn'+IntToStr(dir))).Down:=true;

  {Convert dir to directions for the motors}
  if LeftRight then
  begin
    ldir := ldirA[dir];
    rdir := rdirA[dir];
  end
  else
  begin
    ldir := ldirB[dir];
    rdir := rdirB[dir];
  end;

  if LeftReversed then
    ldir := -ldir;
  if RightReversed then
    rdir := -rdir;

  l  := GetMotorList(LeftMotor);
  r  := GetMotorList(RightMotor);
  lr := GetMotorList(LeftMotor, RightMotor);

  {Execute the motions}
  if MotorSpeed <> oldspeed then
  begin
    FakeRCX.SetMotorPower(lr, 2, MotorSpeed);
  end;

  if (ldir <> oldldir) or (rdir <> oldrdir) then
  begin
    if ldir < 0 then
    begin
      if rdir = 0 then
      begin
        FakeRCX.MotorsOff(r);
        FakeRCX.SetRwd(l);
        FakeRCX.MotorsOn(l);
      end
      else if rdir > 0 then
      begin
        FakeRCX.SetRwd(l);
        FakeRCX.SetFwd(r);
        FakeRCX.MotorsOn(lr);
      end
      else // if rdir < 0 then
      begin
        FakeRCX.SetRwd(lr);
        FakeRCX.MotorsOn(lr);
      end;
    end
    else if ldir > 0 then
    begin
      if rdir = 0 then
      begin
        FakeRCX.MotorsOff(r);
        FakeRCX.SetFwd(l);
        FakeRCX.MotorsOn(l);
      end
      else if rdir > 0 then
      begin
        FakeRCX.SetFwd(lr);
        FakeRCX.MotorsOn(lr);
      end
      else // if rdir < 0 then
      begin
        FakeRCX.SetFwd(l);
        FakeRCX.SetRwd(r);
        FakeRCX.MotorsOn(lr);
      end;
    end
    else
    begin
      if rdir = 0 then
      begin
        FakeRCX.MotorsOff(lr);
      end
      else if rdir > 0 then
      begin
        FakeRCX.MotorsOff(l);
        FakeRCX.SetFwd(r);
        FakeRCX.MotorsOn(r);
      end
      else // if rdir < 0 then
      begin
        FakeRCX.MotorsOff(l);
        FakeRCX.SetRwd(r);
        FakeRCX.MotorsOn(r);
      end;
    end;
  end;

  {Save the values}
  oldldir := ldir;
  oldrdir := rdir;
  oldspeed := MotorSpeed;
end;

var oldjoydir:integer;

procedure TJoystickForm.JoyTimerTimer(Sender: TObject);
{ The timer callback that does the actual work}
var
  joyinfo : TJoyInfo;
  buttons : array [1..4] of boolean;
  i : integer;
  dir : integer;
begin
  {Get the joystick value and check for errors}
  if joyGetPos(JOYSTICKID1,@joyinfo) <> JOYERR_NOERROR then
    Exit;

  {Handle the position}
  if (joyinfo.wYpos<20000) then      dir:=6
  else if (joyinfo.wYpos>44000) then dir:=0
  else                               dir:=3;
  if (joyinfo.wXpos<20000) then      dir:=dir+1
  else if (joyinfo.wXpos>44000) then dir:=dir+3
  else                               dir:=dir+2;
  if dir <> oldjoydir then MoveRCX(dir);
  oldjoydir := dir;

  {Handle the buttons}
  buttons[1] := (JOY_BUTTON1 and joyinfo.wButtons) > 0;
  buttons[2] := (JOY_BUTTON2 and joyinfo.wButtons) > 0;
  buttons[3] := (JOY_BUTTON3 and joyinfo.wButtons) > 0;
  buttons[4] := (JOY_BUTTON4 and joyinfo.wButtons) > 0;
  for i:=1 to 4 do
    if buttons[i] <> oldbuttons[i] then
    begin
      oldbuttons[i] := buttons[i];
      if buttons[i] then
      begin
        FakeRCX.StartTask(i);
      end;
    end;
end;

procedure TJoystickForm.DirBtnMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MoveRCX(TSpeedButton(Sender).Tag);
end;

procedure TJoystickForm.DirBtnMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button <> mbRight then MoveRCX(5);
end;

procedure TJoystickForm.DirBtnClick(Sender: TObject);
begin
  MoveRCX(5);
end;

procedure TJoystickForm.TaskBtnClick(Sender: TObject);
begin
  FakeRCX.StartTask(TBitBtn(Sender).Tag);
end;

procedure TJoystickForm.LeftRightBtnClick(Sender: TObject);
begin
  LeftRightBtn.Checked := true;
  LeftGroup.Caption := ' Left Motor ';
  RightGroup.Caption := ' Right Motor ';
  LeftRight := true;
end;

procedure TJoystickForm.DriveSteerBtnClick(Sender: TObject);
begin
  DriveSteerBtn.Checked := true;
  LeftGroup.Caption := ' Drive Motor ';
  RightGroup.Caption := ' Steer Motor ';
  LeftRight := false;
end;

procedure TJoystickForm.LMotorClick(Sender: TObject);
{Left motor selection}
begin
  LeftMotor := TRadioButton(Sender).Tag;
end;

procedure TJoystickForm.LReversedClick(Sender: TObject);
{Left Reverse selection}
begin
  LeftReversed := LReversed.Checked;
end;

procedure TJoystickForm.RMotorClick(Sender: TObject);
{Right motor selection}
begin
  RightMotor := TRadioButton(Sender).Tag;
end;

procedure TJoystickForm.RReversedClick(Sender: TObject);
{Right reverse selection}
begin
  RightReversed := RReversed.Checked;
end;

procedure TJoystickForm.SpeedBarChange(Sender: TObject);
{Speed selection}
begin
  MotorSpeed := SpeedBar.Position;
end;

procedure TJoystickForm.FormShow(Sender: TObject);
{Form becomes visible}
begin
  {Fill in values}
  if LeftRight then
    LeftRightBtnClick(Self)
  else
    DriveSteerBtnClick(Self);
  LMotorA.Checked := (LeftMotor = 0);
  LMotorB.Checked := (LeftMotor = 1);
  LMotorC.Checked := (LeftMotor = 2);
  RMotorA.Checked := (RightMotor = 0);
  RMotorB.Checked := (RightMotor = 1);
  RMotorC.Checked := (RightMotor = 2);
  LReversed.Checked := LeftReversed;
  RReversed.Checked := RightReversed;
  SpeedBar.Position := MotorSpeed;
  {Start the timer}
  JoyTimer.Enabled := true;
end;

procedure TJoystickForm.FormClose(Sender: TObject; var Action: TCloseAction);
{Form stops from being visible}
begin
  {Stop the timer}
  JoyTimer.Enabled := false;
end;

procedure TJoystickForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key >=VK_NUMPAD1) and (Key <=VK_NUMPAD9) then
    MoveRCX(Key-VK_NUMPAD0);
end;

procedure TJoystickForm.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key >=VK_NUMPAD1) and (Key <=VK_NUMPAD9) then
    MoveRCX(5);
end;

end.
