unit rcx_cmd;

interface

uses
  Classes, SysUtils, Windows;

const
  kRCX_Cmd_MaxShortLength	= 8;

type
  // rcx fragment type
  TRcxFragment = (kRCX_TaskFragment, kRCX_SubFragment);
  // variable code
  TRcxVarCode = (kRCX_SetVar, kRCX_AddVar, kRCX_SubVar, kRCX_DivVar,
    kRCX_MulVar, kRCX_SgnVar, kRCX_AbsVar, kRCX_AndVar, kRCX_OrVar);

  TRcxCmd = class
  private
	  fLength : integer;
    fBodyPtr : PByte;
    fBodyData : array [0..kRCX_Cmd_MaxShortLength] of Byte;
    function MakeValue16(opcode : Byte; value : integer) : TRcxCmd;
	  function MakeValue8(opcode : Byte; value : integer) : TRcxCmd;
    procedure ClearBodyPointer;
  protected
    fLog : string;
    procedure WriteToLog(s : string);
    function DirSpeed(val : integer) : Byte;
  public
    constructor Create;
    destructor Destroy; override;
	  function GetLength : integer;
	  function GetBody : PByte;
	  function CopyOut(dst : PByte) : integer;
    procedure SetLength(length : integer);

	  function SetVal(data : PByte; length : integer) : TRcxCmd; overload;
	  function SetVal(d0 : Byte) : TRcxCmd; overload;
	  function SetVal(d0, d1 : Byte) : TRcxCmd; overload;
	  function SetVal(d0, d1, d2 : Byte) : TRcxCmd; overload;
	  function SetVal(d0, d1, d2, d3 : Byte) : TRcxCmd; overload;
	  function SetVal(d0, d1, d2, d3, d4 : Byte) : TRcxCmd; overload;
	  function SetVal(d0, d1, d2, d3, d4, d5 : Byte) : TRcxCmd; overload;

    // utility functions to create specific commands
	  // variables
	  function  MakeVar(code : TRcxVarCode; myvar : Byte; value : integer) : TRcxCmd;

    // outputs
    function  MakeOutputMode(outputs : Byte; mode : integer) : TRcxCmd;
    function  MakeOutputPower(outputs : Byte; value : integer) : TRcxCmd;
    function  MakeOutputDir(outputs : Byte; dir : integer) : TRcxCmd;

    // inputs
    function 	MakeInputMode(input : Byte; mode : integer) : TRcxCmd;
    function 	MakeInputType(input : Byte; itype : integer) : TRcxCmd;

    // sound
    function  MakePlaySound(sound : Byte) : TRcxCmd;
    function  MakePlayTone(freq : Word; duration : Byte) : TRcxCmd;

    // control flow
    function  MakeTest(v1 : integer; rel : integer; v2 : integer; offset : smallint) : TRcxCmd;
    function  MakeJump(offset : smallint) : TRcxCmd;
    function  MakeSetLoop(v : integer) : TRcxCmd;
    function  MakeCheckLoop(offset : smallint) : TRcxCmd;

    // cybermaster stuff
    function MakeDrive(m0, m1 : integer) : TRcxCmd;
    function MakeOnWait(outputs : Byte; aNum : integer; aTime : Byte) : TRcxCmd;
    function MakeOnWaitDifferent(outputs : Byte; aNum0, aNum1, aNum2 : integer; aTime : Byte) : TRcxCmd;

    // misc
    function  MakeStopTask(task : Byte) : TRcxCmd;
    function  MakeDelay(value : integer) : TRcxCmd;
    function  MakeDisplay(value : integer) : TRcxCmd; overload;
    function  MakeDisplay(src, value : integer) : TRcxCmd; overload;
    function  MakeSet(dst, src : integer) : TRcxCmd;

    // system commands
    function  MakeRead(value : integer) : TRcxCmd;
    function  MakeUnlock : TRcxCmd;
    function  MakeBegin(ftype : TRcxFragment; taskNumber : Byte; length : Word) : TRcxCmd;
    function  MakeDownload(seq : Word; data : PByte; length : Word) : TRcxCmd;
    function  MakeDeleteTasks : TRcxCmd;
    function  MakeDeleteSubs : TRcxCmd;
    function  MakePing : TRcxCmd;
    function  MakeUploadDatalog(start, count : Word) : TRcxCmd;
    function  MakeSetDatalog(size : Word) : TRcxCmd;
    function  MakeBoot : TRcxCmd;

    // special command to unlock CyberMaster
    function  MakeUnlockCM : TRcxCmd;

    procedure SetOffset(offset : integer);
    procedure Print;
    property Log : string read fLog;
  end;

implementation

uses
  rcx_constants;

function RCX_VALUE(t : TRcxValueType; d : smallint) : integer;
begin
  result := integer((t shl 16) or (d and $FFFF));
end;

function RCX_VALUE_TYPE(v : integer) : TRcxValueType;
begin
  result := (v shr 16) and $FF;
end;

function RCX_VALUE_DATA(v : integer) : smallint;
begin
  result := smallint(v and $FFFF);
end;

function kRCX_VarOp(code : TRcxVarCode) : integer;
begin
  result := $14 + (Ord(code) * 16);
end;

{ TRcxCmd }

procedure TRcxCmd.ClearBodyPointer;
begin
  if fLength > kRCX_Cmd_MaxShortLength then
    FreeMem(fBodyPtr, fLength);
end;

function TRcxCmd.CopyOut(dst: PByte): integer;
var
  tmp : PByte;
begin
  tmp := GetBody;
  Move(tmp^, dst^, fLength);
  result := fLength;
end;

constructor TRcxCmd.Create;
begin
  inherited Create;
  fLog := '';
  fLength := 0;
end;

destructor TRcxCmd.Destroy;
begin
  ClearBodyPointer;
  inherited Destroy;
end;

function TRcxCmd.DirSpeed(val: integer): Byte;
begin
  result := ((val) and 8 xor 8 or ((val)*(((val) shr 3) * 2 xor 1)) and 7);
	// Aux. function: Transforms a number between -7 and 7 to a 4 bit sequence:
	// Bits: 1..3 - Speed: 0 to 7
	// Bit : 4    - Direction: 1 if v>=0, 0 if v<0
//	#define DIRSPEED(v)		((v)&8^8|((v)*(((v)>>3)*2^1))&7)

end;

function TRcxCmd.GetBody: PByte;
begin
  if fLength <= kRCX_Cmd_MaxShortLength then
    result := @fBodyData[0]
  else
    result := fBodyPtr;
end;

function TRcxCmd.GetLength: integer;
begin
  result := fLength;
end;

function TRcxCmd.MakeBegin(ftype: TRcxFragment; taskNumber: Byte;
  length: Word): TRcxCmd;
var
  op : Byte;
begin
  if ftype = kRCX_TaskFragment then
    op := kRCX_BeginTaskOp
  else
    op := kRCX_BeginSubOp;
	result := SetVal(op, 0, taskNumber, 0, Lo(length), Hi(length));
end;

function TRcxCmd.MakeBoot: TRcxCmd;
begin
	SetVal(kRCX_UnlockFirmOp, $4c, $45, $47, $4f, $ae);
  // LEGO
	result := self;
end;

function TRcxCmd.MakeCheckLoop(offset: smallint): TRcxCmd;
begin
	SetVal(kRCX_CheckLoopOp, 0, 0);
	SetOffset(offset);
	result := self;
end;

function TRcxCmd.MakeDelay(value: integer): TRcxCmd;
begin
  result := MakeValue16(kRCX_DelayOp, value);
end;

function TRcxCmd.MakeDeleteSubs: TRcxCmd;
begin
  result := SetVal(kRCX_DeleteSubsOp);
end;

function TRcxCmd.MakeDeleteTasks: TRcxCmd;
begin
  result := SetVal(kRCX_DeleteTasksOp);
end;

function TRcxCmd.MakeDisplay(value: integer): TRcxCmd;
begin
  result := MakeValue16(kRCX_DisplayOp, value);
end;

function TRcxCmd.MakeDisplay(src, value: integer): TRcxCmd;
begin
  result := SetVal(kRCX_DisplayOp, Byte(src), Lo(value), Hi(value));
end;

function TRcxCmd.MakeDownload(seq: Word; data: PByte; length: Word): TRcxCmd;
var
  ptr : PByte;
  checksum, b : Byte;
begin
	SetLength(6 + length);

	ptr := GetBody;
  ptr^ := kRCX_DownloadOp;
  inc(ptr);
  ptr^ := Lo(seq);
  inc(ptr);
  ptr^ := Hi(seq);
  inc(ptr);
  ptr^ := Lo(length);
  inc(ptr);
  ptr^ := Hi(length);
  inc(ptr);

	checksum := 0;
	while length > 0 do
  begin
    b := data^;
    inc(data);
		checksum := checksum + b;
    ptr^ := b;
    inc(ptr);
    dec(length);
  end;

  ptr^ := checksum;
	result := self;
end;

function TRcxCmd.MakeDrive(m0, m1: integer): TRcxCmd;
begin
  result := SetVal(kRCX_DriveOp, DirSpeed(m0) or DirSpeed(m1) shl 4);
end;

function TRcxCmd.MakeInputMode(input: Byte; mode: integer): TRcxCmd;
begin
  result := SetVal(kRCX_InputModeOp, input, mode);
end;

function TRcxCmd.MakeInputType(input: Byte; itype: integer): TRcxCmd;
begin
  result := SetVal(kRCX_InputTypeOp, input, itype);
end;

function TRcxCmd.MakeJump(offset: smallint): TRcxCmd;
var
  ptr : PByte;
begin
	SetLength(3);
  ptr := GetBody;
	ptr^ := kRCX_JumpOp;
	SetOffset(offset);
  result := self;
end;

function TRcxCmd.MakeOnWait(outputs: Byte; aNum: integer;
  aTime: Byte): TRcxCmd;
begin
  result := SetVal(kRCX_OnWaitOp, (outputs shl 4) or DirSpeed(aNum), aTime);
end;

function TRcxCmd.MakeOnWaitDifferent(outputs: Byte; aNum0, aNum1,
  aNum2: integer; aTime: Byte): TRcxCmd;
begin
  result := SetVal(kRCX_OnWaitDifferentOp, (outputs shl 4) or DirSpeed(aNum0), (DirSpeed(aNum1) shl 4) or DirSpeed(aNum2), aTime);
end;

function TRcxCmd.MakeOutputDir(outputs: Byte; dir: integer): TRcxCmd;
begin
  result := SetVal(kRCX_OutputDirOp, Byte(dir or outputs));
end;

function TRcxCmd.MakeOutputMode(outputs: Byte; mode: integer): TRcxCmd;
begin
  result := SetVal(kRCX_OutputModeOp, Byte(mode or outputs));
end;

function TRcxCmd.MakeOutputPower(outputs: Byte; value: integer): TRcxCmd;
begin
	result := SetVal(kRCX_OutputPowerOp, outputs, Byte(RCX_VALUE_TYPE(value)), Byte(RCX_VALUE_DATA(value)));
end;

function TRcxCmd.MakePing: TRcxCmd;
begin
  result := SetVal(kRCX_PingOp);
end;

function TRcxCmd.MakePlaySound(sound: Byte): TRcxCmd;
begin
  result := SetVal(kRCX_PlaySoundOp, Byte(sound and 7));
end;

function TRcxCmd.MakePlayTone(freq: Word; duration: Byte): TRcxCmd;
begin
	result := SetVal(kRCX_PlayToneOp, Lo(freq), Hi(freq), duration);
end;

function TRcxCmd.MakeRead(value: integer): TRcxCmd;
begin
  result := MakeValue8(kRCX_ReadOp, value);
end;

function TRcxCmd.MakeSet(dst, src: integer): TRcxCmd;
var
  ptr : PByte;
begin
	SetLength(6);
	ptr := GetBody;

  ptr^ := kRCX_SetSourceValueOp;
  inc(ptr);
  ptr^ := Byte(RCX_VALUE_TYPE(dst));
  inc(ptr);
  ptr^ := Lo(RCX_VALUE_DATA(dst));
  inc(ptr);
  ptr^ := Byte(RCX_VALUE_TYPE(src));
  inc(ptr);
  ptr^ := Lo(RCX_VALUE_DATA(src));
  inc(ptr);
  ptr^ := Hi(RCX_VALUE_DATA(src));
	result := self;
end;

function TRcxCmd.MakeSetDatalog(size: Word): TRcxCmd;
begin
	result := SetVal(kRCX_SetDatalogOp, Lo(size), Hi(size));
end;

function TRcxCmd.MakeSetLoop(v: integer): TRcxCmd;
begin
  result := MakeValue8(kRCX_SetLoopOp, v);
end;

function TRcxCmd.MakeStopTask(task: Byte): TRcxCmd;
begin
  result := SetVal(kRCX_StopTaskOp, task);
end;

function TRcxCmd.MakeTest(v1, rel, v2: integer; offset: smallint): TRcxCmd;
var
  d1 : integer;
  ptr : PByte;
begin
	d1 := RCX_VALUE_DATA(v1);
	SetLength(8);

	ptr := GetBody;

  ptr^ := kRCX_TestOp;
  inc(ptr);
  ptr^ := Byte(((rel shl 6) or Byte(RCX_VALUE_TYPE(v1))));
  inc(ptr);
  ptr^ := Byte(RCX_VALUE_TYPE(v2));
  inc(ptr);
  ptr^ := Lo(d1);
  inc(ptr);
  ptr^ := Hi(d1);
  inc(ptr);
  ptr^ := Byte(RCX_VALUE_DATA(v2));
	SetOffset(offset);
	result := self;
end;

function TRcxCmd.MakeUnlock: TRcxCmd;
begin
  result := SetVal(kRCX_UnlockOp, 1, 3, 5, 7, $b);
end;

type
  MsgType = array[0..26] of Byte;

function TRcxCmd.MakeUnlockCM: TRcxCmd;
const
  unlockMsg : MsgType = (kRCX_UnlockFirmOp,
	  Ord('D'),Ord('o'),Ord(' '),Ord('y'),Ord('o'),Ord('u'),
    Ord(' '),Ord('b'),Ord('y'),Ord('t'),Ord('e'),Ord(','),
    Ord(' '),Ord('w'),Ord('h'),Ord('e'),Ord('n'),Ord(' '),
    Ord('I'),Ord(' '),Ord('k'),Ord('n'),Ord('o'),Ord('c'),
    Ord('k'),Ord('?'));
var
  msg : MsgType;
begin
  msg := unlockMsg;
	SetVal(@msg[0], sizeof(unlockMsg));
	result := self;
end;

function TRcxCmd.MakeUploadDatalog(start, count: Word): TRcxCmd;
begin
	result := SetVal(kRCX_UploadDatalogOp,
    Lo(start), Hi(start),
		Lo(count), Hi(count));
end;

function TRcxCmd.MakeValue16(opcode: Byte; value: integer): TRcxCmd;
var
  data : integer;
begin
	data := RCX_VALUE_DATA(value);
	result := SetVal(opcode, Byte(RCX_VALUE_TYPE(value)), Lo(data), Hi(data));
end;

function TRcxCmd.MakeValue8(opcode: Byte; value: integer): TRcxCmd;
begin
	result := SetVal(opcode, Byte(RCX_VALUE_TYPE(value)), Byte(RCX_VALUE_DATA(value)));
end;

function TRcxCmd.MakeVar(code: TRcxVarCode; myvar: Byte; value: integer): TRcxCmd;
var
  data : integer;
begin
	data := RCX_VALUE_DATA(value);
	result := SetVal(Byte(kRCX_VarOp(code)), myvar, Byte(RCX_VALUE_TYPE(value)),
    Lo(data), Hi(data));
end;

procedure TRcxCmd.Print;
var
  ptr : PByte;
  i : integer;
  tmp : string;
begin
	ptr := GetBody;

  tmp := '';
	for i := 0 to fLength-1 do
  begin
    tmp := tmp + Format('%2x', [ptr^]);
    inc(ptr);
  end;
  WriteToLog(tmp + #13#10);
end;

procedure TRcxCmd.SetLength(length: integer);
begin
  if length = fLength then Exit;
  ClearBodyPointer;

  fLength := length;

  if fLength > kRCX_Cmd_MaxShortLength then
    fBodyPtr := AllocMem(fLength * sizeof(Byte));
end;

procedure TRcxCmd.SetOffset(offset: integer);
var
  ptr, tmp : PByte;
  neg : Byte;
begin
	neg := 0;

	offset := offset - (fLength - 2);
	ptr := GetBody;
  inc(ptr, fLength);
  dec(ptr, 2);

  tmp := GetBody;
	if tmp^ <> kRCX_JumpOp then
  begin
    ptr^ := Byte(offset and $FF);
    inc(ptr);
    ptr^ := Byte((offset shr 8) and $FF);
  end
	else
  begin
		if offset < 0 then
    begin
			neg := $80;
			offset := -offset;
    end;
    ptr^ := Byte(neg or (offset and $7F));
    inc(ptr);
    ptr^ := Byte((offset shr 7) and $FF);
  end;
end;

function TRcxCmd.SetVal(d0, d1, d2, d3: Byte): TRcxCmd;
begin
  SetLength(4);
  fBodyData[0] := d0;
  fBodyData[1] := d1;
  fBodyData[2] := d2;
  fBodyData[3] := d3;
  result := self;
end;

function TRcxCmd.SetVal(d0, d1, d2, d3, d4: Byte): TRcxCmd;
begin
  SetLength(5);
  fBodyData[0] := d0;
  fBodyData[1] := d1;
  fBodyData[2] := d2;
  fBodyData[3] := d3;
  fBodyData[4] := d4;
  result := self;
end;

function TRcxCmd.SetVal(d0, d1, d2, d3, d4, d5: Byte): TRcxCmd;
begin
  SetLength(6);
  fBodyData[0] := d0;
  fBodyData[1] := d1;
  fBodyData[2] := d2;
  fBodyData[3] := d3;
  fBodyData[4] := d4;
  fBodyData[5] := d5;
  result := self;
end;

function TRcxCmd.SetVal(d0, d1, d2: Byte): TRcxCmd;
begin
  SetLength(3);
  fBodyData[0] := d0;
  fBodyData[1] := d1;
  fBodyData[2] := d2;
  result := self;
end;

function TRcxCmd.SetVal(data: PByte; length: integer): TRcxCmd;
var
  tmp : PByte;
begin
  SetLength(length);
  tmp := GetBody;
  Move(data^, tmp^, length);
  result := self;
end;

function TRcxCmd.SetVal(d0: Byte): TRcxCmd;
begin
  SetLength(1);
  fBodyData[0] := d0;
  result := self;
end;

function TRcxCmd.SetVal(d0, d1: Byte): TRcxCmd;
begin
  SetLength(2);
  fBodyData[0] := d0;
  fBodyData[1] := d1;
  result := self;
end;

procedure TRcxCmd.WriteToLog(s: string);
begin
  fLog := fLog + s;
end;

end.
