unit SearchRCX;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TSearchRCXForm = class(TForm)
    Label1: TLabel;
    OKBtn: TButton;
    CancelBtn: TButton;
    pnlPorts: TPanel;
    AutomaticBtn: TRadioButton;
    COM1Btn: TRadioButton;
    COM2Btn: TRadioButton;
    COM3Btn: TRadioButton;
    COM4Btn: TRadioButton;
    Panel2: TPanel;
    RCXBtn: TRadioButton;
    CyberMasterBtn: TRadioButton;
    ScoutBtn: TRadioButton;
    Rcx2Btn: TRadioButton;
    USB1Btn: TRadioButton;
    USB4Btn: TRadioButton;
    USB3Btn: TRadioButton;
    USB2Btn: TRadioButton;
    COM5Btn: TRadioButton;
    COM6Btn: TRadioButton;
    COM7Btn: TRadioButton;
    COM8Btn: TRadioButton;
  private
    { Private declarations }
  public
    { Public declarations }
    function GetPortNumber : integer;
    function GetRCXType : integer;
    procedure SetPortNumber(aNum : integer);
    procedure SetRCXType(aType : integer);
  end;

var
  SearchRCXForm: TSearchRCXForm;

procedure SearchForRCX(atstartup:boolean);

function CheckAlive : boolean; //Checks whether the RCX is still alive
function IsRCX2 : boolean;
function IsRCX : boolean;
function IsScout : boolean;
function UseUSB : Boolean;

var
  IRexists:boolean;          // Whether the COM port exists


implementation

{$R *.DFM}

uses
  Preferences, FakeSpirit, rcx_link;

resourcestring
  S_CANNOT_FIND_RCX = 'Cannot find brick. Switch it on or move it closer and press OK.';

function IsRCX2 : boolean;
begin
  result := LocalRCXType = SU_RCX2;
end;

function IsRCX : boolean;
begin
  result := (LocalRCXType = SU_RCX) or (LocalRCXType = SU_RCX2);
end;

function IsScout : boolean;
begin
  result := (LocalRCXType = SU_SCOUT);
end;

function UseUSB : Boolean;
begin
  Result := PortIsUSB(LocalPortNum);
end;

{Checks whether the RCX is (still) alive}
function CheckAlive : boolean;
begin
  result := false;
  // always start with a closed FakeRCX
  FakeRCX.Close;
  while true do
  begin
    if FakeRCX.BrickAlive then
      break
    else if MessageDlg(S_CANNOT_FIND_RCX, mtWarning, [mbOK, mbCancel], 0) = mrCancel then
      exit;
  end;
  result:=true;
end;

{Sets the type of the RCX}
procedure SetRCXType(thetype:integer);
begin
  FakeRCX.RCXType := thetype;
end;

{Searches for the COM port}
function FindPort(theport:integer):boolean;
var i,pmin,pmax:integer;
begin
  IRexists:=false;
  if theport = 0 then
  begin
    pmin:=Low(TPortNum);
    pmax:=High(TPortNum);
  end
  else
  begin
    pmin:=theport;
    pmax:=theport;
  end;
  for i:=pmin to pmax do
  begin
    FakeRCX.PortNum := i;
    if FakeRCX.TowerExists then
    begin
      IRexists:=true;
      LocalPortNum := i;
      break;
    end;
  end;
  result := IRexists;
end;

{Does the actual searching}
function FindIt(thetype,theport:integer):boolean;
begin
  SetRCXType(thetype);
  FindPort(theport);
  {If it exists, find the RCX}
  if IRexists then
    result := CheckAlive
  else
    result := false;
end;

{Tries to locate the RCX. atstartup indicates whether this is the
 call at startup, in which case we use the startup defaults.}
procedure SearchForRCX(atstartup:boolean);
begin
  IRexists:=false;
  if (atstartup and (LocalStartupAction = SU_NOCONNECT)) then
    Exit;
  if (atstartup and (LocalStartupAction = SU_CONNECT)) or
     (not (atstartup or FBAlwaysPrompt)) then
  begin
    if FindIt(LocalRCXType,LocalPortNum) then Exit;
  end;
  SearchRCXForm.SetPortNumber(LocalPortNum);
  SearchRCXForm.SetRCXType(LocalRCXType);
  if SearchRCXForm.ShowModal = mrOk then
  begin
    LocalPortNum := SearchRCXForm.GetPortNumber;
    LocalRCXType := SearchRCXForm.GetRCXType;
    FindIt(LocalRCXType,LocalPortNum);
  end;
end;

{ TSearchRCXForm }

function TSearchRCXForm.GetPortNumber: integer;
var
  i : integer;
  RB : TRadioButton;
begin
  result := 0;
  for i := 0 to pnlPorts.ControlCount - 1 do
  begin
    if pnlPorts.Controls[i] is TRadioButton then
    begin
      RB := TRadioButton(pnlPorts.Controls[i]);
      if RB.Checked then
      begin
        result := RB.Tag;
        exit;
      end;
    end;
  end;
end;

function TSearchRCXForm.GetRCXType: integer;
begin
  result := SU_RCX;
  if CyberMasterBtn.Checked then
    result := SU_CYBERMASTER
  else if ScoutBtn.Checked then
    result := SU_SCOUT
  else if Rcx2Btn.Checked then
    result := SU_RCX2;
end;

procedure TSearchRCXForm.SetPortNumber(aNum: integer);
var
  i : integer;
  RB : TRadioButton;
begin
  for i := 0 to pnlPorts.ControlCount - 1 do
  begin
    if pnlPorts.Controls[i] is TRadioButton then
    begin
      RB := TRadioButton(pnlPorts.Controls[i]);
      if RB.Tag = aNum then
      begin
        RB.Checked := True;
        exit;
      end;
    end;
  end;
end;

procedure TSearchRCXForm.SetRCXType(aType: integer);
begin
  case aType of
    SU_CYBERMASTER : CyberMasterBtn.Checked := True;
    SU_SCOUT       : ScoutBtn.Checked       := True;
    SU_RCX2        : Rcx2Btn.Checked        := True;
  else
    RCXBtn.Checked := True;
  end;
end;

end.
