unit Transdlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls, Buttons;

type
  TfrmTransEdit = class(TForm)
    lblTitle: TLabel;
    lblProgram: TLabel;
    lblParameters: TLabel;
    lblMacros: TLabel;
    lblWorkingDir: TLabel;
    edTitle: TEdit;
    edProgram: TEdit;
    edWorkingDir: TEdit;
    edParameters: TEdit;
    MacroButton: TBitBtn;
    MacroList: TListBox;
    OKButton: TButton;
    CancelButton: TButton;
    InsertButton: TButton;
    BrowseButton: TButton;
    OpenDialog: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MacroButtonClick(Sender: TObject);
    procedure BrowseButtonClick(Sender: TObject);
    procedure InsertButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure edTitleChange(Sender: TObject);
    procedure MacroListClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
  private
    { Private declarations }
    bmpUp, bmpDn : TBitmap;
    fCaptionHeight : integer;
    procedure SetSize(bLarge : boolean = false);
    function SizeIsLarge : boolean;
    function GetMacroGlyph : TBitmap;
    procedure UpdateButtonState;
    procedure LoadMacros;
  public
    { Public declarations }
  end;

var
  frmTransEdit: TfrmTransEdit;
  TransferMacros : array[0..10] of String =
    ('$COL', '$ROW', '$CURTOKEN', '$PATH()', '$NAME()', '$NAMEONLY()',
     '$EXT()', '$EDNAME', '$PROMPT()', '$SAVE', '$SAVEALL');

implementation

{$R *.DFM}

const
  SMALL_SIZE = 160;
  LARGE_SIZE = 282;

procedure TfrmTransEdit.FormCreate(Sender: TObject);
begin
  LoadMacros;
  fCaptionHeight := GetSystemMetrics(SM_CYCAPTION);
  bmpUp := TBitmap.Create;
  bmpDn := TBitmap.Create;
  bmpUp.LoadFromResourceName(HInstance, 'MACROUP');
  bmpDn.LoadFromResourceName(HInstance, 'MACRODN');
  SetSize;
  UpdateButtonState;
end;

procedure TfrmTransEdit.FormDestroy(Sender: TObject);
begin
  bmpUp.Free;
  bmpDn.Free;
end;

procedure TfrmTransEdit.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
//
end;

procedure TfrmTransEdit.MacroButtonClick(Sender: TObject);
begin
  SetSize(not SizeIsLarge);
end;

procedure TfrmTransEdit.BrowseButtonClick(Sender: TObject);
begin
  if OpenDialog.Execute then
  begin
    edProgram.Text := OpenDialog.FileName;
    edWorkingDir.Text := ExtractFileDir(OpenDialog.FileName);
  end;
end;

procedure TfrmTransEdit.InsertButtonClick(Sender: TObject);
begin
  if MacroList.ItemIndex <> -1 then
  begin
    edParameters.SelText := TransferMacros[MacroList.ItemIndex];
  end;
end;

procedure TfrmTransEdit.SetSize(bLarge: boolean);
var
  h : integer;
begin
  if bLarge then
    h := LARGE_SIZE
  else
    h := SMALL_SIZE;
  Height := h + fCaptionHeight;
  MacroButton.Glyph := GetMacroGlyph;
end;

function TfrmTransEdit.SizeIsLarge: boolean;
begin
  result := Height = LARGE_SIZE + fCaptionHeight;
end;

function TfrmTransEdit.GetMacroGlyph: TBitmap;
begin
  result := bmpDn;
  if SizeIsLarge then
    result := bmpUp;
end;

procedure TfrmTransEdit.edTitleChange(Sender: TObject);
begin
  UpdateButtonState;
end;

procedure TfrmTransEdit.UpdateButtonState;
begin
  OKButton.Enabled := not ((edTitle.Text = '') or (edProgram.Text = ''));
  InsertButton.Enabled := MacroList.ItemIndex <> -1;
end;

procedure TfrmTransEdit.MacroListClick(Sender: TObject);
begin
  UpdateButtonState;
end;

procedure TfrmTransEdit.LoadMacros;
resourcestring
  sColMacro = 'Cursor column in active editor';
  sRowMacro = 'Cursor row in active editor';
  sCurTokenMacro = 'Word at cursor in active editor';
  sPathMacro = 'Directory portion of parameter';
  sNameMacro = 'File name of parameter';
  sNameOnlyMacro = 'File name of parameter without extension';
  sExtMacro = 'File extension of parameter';
  sEdNameMacro = 'Expanded file name of active editor';
  sPromptMacro = 'Prompt for information';
  sSaveMacro = 'Save file in active editor';
  sSaveAllMacro = 'Save all modified files';
begin
  MacroList.Items.Add(TransferMacros[0]+#9+sColMacro);
  MacroList.Items.Add(TransferMacros[1]+#9+sRowMacro);
  MacroList.Items.Add(TransferMacros[2]+#9+sCurTokenMacro);
  MacroList.Items.Add(TransferMacros[3]+#9+sPathMacro);
  MacroList.Items.Add(TransferMacros[4]+#9+sNameMacro);
  MacroList.Items.Add(TransferMacros[5]+#9+sNameOnlyMacro);
  MacroList.Items.Add(TransferMacros[6]+#9+sExtMacro);
  MacroList.Items.Add(TransferMacros[7]+#9+sEdNameMacro);
  MacroList.Items.Add(TransferMacros[8]+#9+sPromptMacro);
  MacroList.Items.Add(TransferMacros[9]+#9+sSaveMacro);
  MacroList.Items.Add(TransferMacros[10]+#9+sSaveAllMacro);
end;

procedure TfrmTransEdit.OKButtonClick(Sender: TObject);
begin
// verify correctness of parameters
end;

end.

