unit srecord;

interface

uses
  Windows, sysutils;

type
  TSRecord = class
  private
    fLength : integer;
    fStart : integer;
    fData : PByte;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    function GetLength : integer;
    function GetStart : integer;
    function GetData : PByte;
    function Read(var F : TextFile; maxLength : integer) : boolean;

	  class function ReadHexByte(const ptr : PChar) : integer;
  end;

implementation

{ TSRecord }

const
  kMaxLine = 256;

constructor TSRecord.Create;
begin
  inherited Create;
  fLength := 0;
  fStart  := 0;
  fData   := nil;
end;

destructor TSRecord.Destroy;
begin
  if fData <> nil then
    FreeMem(fData, fLength);
  inherited Destroy;
end;

function TSRecord.GetData: PByte;
begin
  result := fData;
end;

function TSRecord.GetLength: integer;
begin
  result := fLength;
end;

function TSRecord.GetStart: integer;
begin
  result := fStart;
end;

function TSRecord.Read(var F : TextFile; maxLength: integer): boolean;
var
  line : array[0..kMaxLine-1] of Char;
  ptr, ptr2 : PChar;
  i, len : integer;
  dataLen : integer;
  byte : integer;
begin
  result := False;
	len := 0;
  if fData <> nil then
    FreeMem(fData, fLength);

  fData := AllocMem(maxLength * sizeof(Byte));
	fLength := 0;

  while not Eof(F) do
  begin
    System.ReadLn(F, line);
		ptr := line;
    // hack for DOS line endings
		if (ptr^ = Chr(10)) or (ptr^ = Chr(13)) then
			inc(ptr);

		if ptr^ = Chr(0) then Continue;

		if ptr^ <> 'S' then Exit;

    ptr2 := ptr;
    inc(ptr2);
		if ptr2^ = '1' then
		begin
			// data line
			dataLen := ReadHexByte(ptr + 2);
			if dataLen < 3 then Exit;

      inc(ptr, 8);
			for i := 0 to dataLen - 4 do
			begin
				if len = maxLength then Exit;
				byte := ReadHexByte(ptr);
        inc(ptr, 2);
				if byte = -1 then Exit;
        PByte(Integer(fData) + len)^ := byte;
        inc(len);
			end;
		end
		else if ptr2^ = '9' then
		begin
			// start address
			dataLen := ReadHexByte(ptr + 2);
			if dataLen <> 3 then Exit;
			fStart := (ReadHexByte(ptr+4) shl 8) + ReadHexByte(ptr+6);
		end;
	end;
	fLength := len;
	result := true;
end;

class function TSRecord.ReadHexByte(const ptr: PChar): integer;
var
  i, v, n : integer;
  c : Char;
  tmpP : PChar;
begin
  v := 0;
  tmpP := ptr;
  for i := 0 to 1 do
  begin
    c := tmpP^;
		inc(tmpP);

		if (c >= '0') and (c <= '9') then
    begin
			n := Ord(c) - Ord('0');
    end
		else if (c >= 'A') and (c <= 'F') then
    begin
			n := Ord(c) - Ord('A') + 10;
    end
		else if (c >= 'a') and (c <= 'f') then
    begin
			n := Ord(c) - Ord('a') + 10;
    end
		else
    begin
      result := -1;
      Exit;
    end;
		v := (v shl 4) + n;
	end;
	result := v;
end;

end.
