unit uCodeExplorer;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, uNQCProcLexer, ExtCtrls, ImgList, Menus;

type
  TfrmCodeExplorer = class(TForm)
    treCodeExplorer: TTreeView;
    tmrCodeExplorer: TTimer;
    ilCodeExplorer: TImageList;
    popCodeExplorer: TPopupMenu;
    mniCodeExpProp: TMenuItem;
    N1: TMenuItem;
    mniViewEditor: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure treCodeExplorerDblClick(Sender: TObject);
    procedure tmrCodeExplorerTimer(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure treCodeExplorerKeyPress(Sender: TObject; var Key: Char);
    procedure mniCodeExpPropClick(Sender: TObject);
    procedure mniViewEditorClick(Sender: TObject);
  private
    function GetProcessing: Boolean;
    procedure SetProcessing(const Value: Boolean);
  private
    { Private declarations }
    fChangedSinceLastProcess : Boolean;
    nodeS, nodeF, nodeT : TTreeNode;
    fFileName: string;
    fProcessing : Boolean;
    fTopNodeIdx, fSelNodeIdx : Integer;
    procedure HandleGetProcsDone(Sender: TObject);
    procedure CreateNode(aStr : string);
    procedure FocusEditor;
    procedure GotoCode;
    procedure RefreshEntireTree;
    procedure ReprocessFile;
    function GetNodeIndex(aNode : TTreeNode) : Integer;
    function FindNode(aIdx : Integer) : TTreeNode;
    property Processing : Boolean read GetProcessing write SetProcessing;
  public
    { Public declarations }
    procedure ClearTree;
    procedure ProcessFile(sName : string);
    property ChangedSinceLastProcess : Boolean
      read fChangedSinceLastProcess write fChangedSinceLastProcess;
  end;

var
  frmCodeExplorer: TfrmCodeExplorer;

implementation

{$R *.DFM}

uses
  MainUnit, uExplorerOptions, Preferences;

procedure TfrmCodeExplorer.ClearTree;
var
  i : Integer;
begin
  treCodeExplorer.Items.BeginUpdate;
  try
    fTopNodeIdx := GetNodeIndex(treCodeExplorer.TopItem);
    fSelNodeIdx := GetNodeIndex(treCodeExplorer.Selected);
    if Assigned(nodeF) then
      for i := nodeF.Count - 1 downto 0 do
        nodeF.Item[i].Delete;
    if Assigned(nodeS) then
      for i := nodeS.Count - 1 downto 0 do
        nodeS.Item[i].Delete;
    if Assigned(nodeT) then
      for i := nodeT.Count - 1 downto 0 do
        nodeT.Item[i].Delete;
  finally
    treCodeExplorer.Items.EndUpdate;
  end;
end;

procedure TfrmCodeExplorer.CreateNode(aStr: string);
var
  pType, pName, pLine : string;
  i : Integer;
  N : TTreeNode;
begin
  N := nil;
  i := Pos('|', aStr);
  pLine := Copy(aStr, 1, i - 1);
  Delete(aStr, 1, i);
  i := Pos('|', aStr);
  pType := Copy(aStr, 1, i - 1);
  Delete(aStr, 1, i);
  pName := aStr;
  if (pType = 'sub') and Assigned(nodeS) then begin
    N := treCodeExplorer.Items.AddChild(nodeS, pName);
    N.ImageIndex := 1;
    N.SelectedIndex := 4;
  end
  else if (pType = 'task') and Assigned(nodeT) then begin
    N := treCodeExplorer.Items.AddChild(nodeT, pName);
    N.ImageIndex := 2;
    N.SelectedIndex := 5;
  end
  else if Assigned(nodeF) then begin
    N := treCodeExplorer.Items.AddChild(nodeF, pName);
    N.ImageIndex := 3;
    N.SelectedIndex := 6;
  end;
  if Assigned(N) then
    N.Data := Pointer(StrToIntDef(pLine, 0) + 1);
end;

procedure TfrmCodeExplorer.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  MainForm.splCodeExplorer.Visible := False;
  MainForm.pnlCodeExplorer.Visible := False;
  tmrCodeExplorer.Enabled := False;
end;

procedure TfrmCodeExplorer.FormCreate(Sender: TObject);
begin
  fProcessing := False;
  Caption := sExploring;
  ManualDock(MainForm.pnlCodeExplorer);
//  treCodeExplorer.DoubleBuffered := True;
  RefreshEntireTree;
end;

procedure TfrmCodeExplorer.HandleGetProcsDone(Sender: TObject);
var
  S : TStrings;
  i : Integer;
  p : string;
  fThread : TNQCProcLexer;
  N : TTreeNode;
begin
  if not (csDestroying in ComponentState) and (Sender is TNQCProcLexer) then
  begin
    fThread := TNQCProcLexer(Sender);
    fThread.OnTerminate := nil;
    // build nodes in tree view
    try
      S := TStringList.Create;
      try
        S.Text := fThread.Results;
        for i := 0 to S.Count - 1 do
        begin
          p := S[i];
          CreateNode(p);
        end;
      finally
        S.Free;
      end;
      if Assigned(nodeF) then
        nodeF.Expanded := CodeExplorerSettings.FunctionsExpand;
      if Assigned(nodeS) then
        nodeS.Expanded := CodeExplorerSettings.SubroutinesExpand;
      if Assigned(nodeT) then
        nodeT.Expanded := CodeExplorerSettings.TasksExpand;
      fChangedSinceLastProcess := False;
      if CodeExplorerSettings.UseAlphaSort then
      begin
        if Assigned(nodeF) then
          nodeF.AlphaSort;
        if Assigned(nodeS) then
          nodeS.AlphaSort;
        if Assigned(nodeT) then
          nodeT.AlphaSort;
      end;
      N := FindNode(fSelNodeIdx);
      if Assigned(N) then
        treCodeExplorer.Selected := N;
      N := FindNode(fTopNodeIdx);
      if Assigned(N) then
        treCodeExplorer.TopItem := N;
    finally
      Processing := False;
    end;
  end;
end;

procedure TfrmCodeExplorer.ProcessFile(sName : string);
begin
  fFileName := sName;
  ReprocessFile;
end;

procedure TfrmCodeExplorer.treCodeExplorerDblClick(Sender: TObject);
begin
  GotoCode;
end;

procedure TfrmCodeExplorer.tmrCodeExplorerTimer(Sender: TObject);
begin
  if fChangedSinceLastProcess then
    ProcessFile(fFileName);
end;

procedure TfrmCodeExplorer.FormShow(Sender: TObject);
begin
  tmrCodeExplorer.Enabled := True;
end;

procedure TfrmCodeExplorer.FocusEditor;
begin
  if Assigned(MainForm) and (MainForm.ActiveEditorForm <> nil) then
  begin
    Windows.SetFocus(MainForm.ActiveEditorForm.Handle);
  end;
end;

procedure TfrmCodeExplorer.treCodeExplorerKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key = #13 then GotoCode;
end;

procedure TfrmCodeExplorer.GotoCode;
var
  i : Integer;
  N : TTreeNode;
begin
  N := treCodeExplorer.Selected;
  if Assigned(N) and Assigned(N.Data) and
     Assigned(MainForm) and (MainForm.ActiveEditorForm <> nil) then
  begin
    i := Integer(N.Data);
    with MainForm.ActiveEditorForm.TheEditor do
    begin
      CaretXY := Point(1, i);
      EnsureCursorPosVisibleEx(True);
    end;
    FocusEditor;
  end;
end;

procedure TfrmCodeExplorer.mniCodeExpPropClick(Sender: TObject);
begin
  with TfrmExplorerOptions.Create(nil) do
  try
    // set up form
    ConfigureForm(CodeExplorerSettings);
    if ShowModal = mrOk then
    begin
      // read values from form
      UpdateDefaults(CodeExplorerSettings);
      RefreshEntireTree;
    end;
  finally
    Free;
  end;
end;

procedure TfrmCodeExplorer.RefreshEntireTree;
var
  ItemOrder : string;
  i, p : integer;
  s : string;
begin
  ItemOrder := CodeExplorerSettings.CategorySort;
  if ItemOrder = '' then
    ItemOrder := sFunctions + ';' + sSubroutines + ';' + sTasks;
  with treCodeExplorer.Items do
  begin
    BeginUpdate;
    try
      Clear;
      for i := 0 to 2 do begin
        p := Pos(';', ItemOrder);
        if p = 0 then p := Length(ItemOrder) + 1;
        s := Copy(ItemOrder, 1, p-1);
        System.Delete(ItemOrder, 1, p);
        if s = sFunctions then begin
          if CodeExplorerSettings.FunctionsVisible then
            nodeF := Add(nil, s)
          else
            nodeF := nil;
        end
        else if s = sSubroutines then begin
          if CodeExplorerSettings.SubroutinesVisible then
            nodeS := Add(nil, s)
          else
            nodeS := nil;
        end
        else begin
          if CodeExplorerSettings.TasksVisible then
            nodeT := Add(nil, s)
          else
            nodeT := nil;
        end;
      end;
    finally
      EndUpdate;
    end;
  end;
  ReprocessFile;
end;

procedure TfrmCodeExplorer.ReprocessFile;
var
  fGetProcs : TNQCProcLexer;
begin
  if not Assigned(MainForm) then Exit;
  if Processing then Exit;
  Caption := sExploring + ' ' + fFileName;
  Processing := True;
  ClearTree;
  if MainForm.ActiveEditorForm <> nil then
  begin
    if (MainForm.ActiveEditorForm.TheEditor.Lines.Count = 0) or
       (MainForm.ActiveEditorForm.TheEditor.Highlighter <> MainForm.SynNQCSyn) then
    begin
      Processing := False;
      Exit;
    end;
    fGetProcs := TNQCProcLexer.Create(True);
    fGetProcs.FreeOnTerminate := True;
    fGetProcs.OnTerminate := HandleGetProcsDone;
    fGetProcs.TextToParse := MainForm.ActiveEditorForm.TheEditor.Lines.Text;
    fGetProcs.Timeout     := ParseTimeout; // ten seconds
    fGetProcs.Resume;
  end
  else
  begin
    Processing := False;
  end;
end;

procedure TfrmCodeExplorer.mniViewEditorClick(Sender: TObject);
begin
  FocusEditor;
end;

function TfrmCodeExplorer.GetProcessing: Boolean;
begin
  Result := fProcessing;
end;

procedure TfrmCodeExplorer.SetProcessing(const Value: Boolean);
begin
  if Value <> fProcessing then
  begin
    fProcessing := Value;
    if Value then
      treCodeExplorer.Items.BeginUpdate
    else
      treCodeExplorer.Items.EndUpdate;
  end;
end;

function TfrmCodeExplorer.FindNode(aIdx: Integer): TTreeNode;
begin
  Result := nil;
  if (aIdx >= 0) and (aIdx < treCodeExplorer.Items.Count) then
    Result := treCodeExplorer.Items[aIdx];
end;

function TfrmCodeExplorer.GetNodeIndex(aNode: TTreeNode): Integer;
begin
  Result := -1;
  if Assigned(aNode) then
    Result := aNode.AbsoluteIndex;
end;

end.
