unit uVersionInfo;

interface

type
  TVersionAttributes = record
  	PrivateBuild : Boolean;
     SpecialBuild : Boolean;
     PreRelease   : Boolean;
     Debug			 : Boolean;
     InfoInferred : Boolean;
     Patched      : Boolean;
  end;

	TVersionInfo = record
  	CompanyName: 			String;
    FileDescription:  String;
    FileVersion:			String;
    InternalName:			String;
    LegalCopyright:		String;
    OriginalFileName:	String;
    ProductName:			String;
    ProductVersion:		String;
    Comments:         String;
    FileFlags : 			TVersionAttributes;
  end;

function GetVersionInfo(FileName: String): TVersionInfo;

implementation

uses
  Windows, SysUtils;

function GetVersionInfo(FileName: String): TVersionInfo;
var
  PBlock : Pointer;
  BlockSize : Integer;
  Dummy : DWORD;
  Name : Array[0..1024] of Char;
  V : TVersionInfo;

	function VerValue(P: Pointer;Value: String): String;
	var
	  Dummy : UINT;
	  Buffer : Pointer;
	  S : String;
	  B : Array[0..1024] of Char;
    T : Array[0..1024] of Char;
	begin
		S := '\\StringFileInfo\\040904E4\\' + Value ;
	  StrPCopy(B,S);
    Buffer := @T[1] ;
		VerQueryValue(P,B,Buffer,Dummy);
		Result := StrPas(Buffer);
	end;

  function AnalyzeFlags(const AFileFlags : Integer) : TVersionAttributes;
  var sflag,dflag, pflag, rflag, vflag, iflag : integer;
      temp : TVersionAttributes;
  begin
   	temp.SpecialBuild := False;
    temp.PrivateBuild := False;
    temp.PreRelease := False;
    temp.Debug := False;
    temp.InfoInferred := False;
    temp.Patched := False;

     // and returns type according to operand types
 		sflag := AFileFlags and VS_FF_SPECIALBUILD;
    dflag := AFileFlags and VS_FF_DEBUG;
    rflag := AFileFlags and VS_FF_PRERELEASE;
    pflag := AFileFlags and VS_FF_PATCHED;
    vflag := AFileFlags and VS_FF_PRIVATEBUILD;
		iflag := AFileFlags and VS_FF_INFOINFERRED;

    if (sflag = VS_FF_SPECIALBUILD) then
     	temp.SpecialBuild := True;

    if (dflag = VS_FF_DEBUG) then
 			temp.Debug := True;

    if (rflag = VS_FF_PRERELEASE) then
			temp.PreRelease := True;

    if (pflag = VS_FF_PATCHED) then
        temp.Patched := True;

    if (vflag = VS_FF_PRIVATEBUILD) then
			temp.PrivateBuild := True;

    if (iflag = VS_FF_INFOINFERRED) then
     	temp.InfoInferred := True;

    Result := temp;
  end;

  // get the fixed file attribute -FileFlags
  function VerFixedValue(P: Pointer): TVersionAttributes;
	var
    Dummy : UINT;
	  Buffer : Pointer;
    sts : Boolean;
	  S :	String;
	  B :	Array[0..1024] of Char;
    T :	Array[0..1024] of Char;
    Fl : DWord;
    V : PVSFixedFileInfo;
	begin
    S := '\\';
	  StrPCopy(B,S);
    Buffer := @T[1] ;
    sts := VerQueryValue(P,B,Buffer,Dummy);

	  if sts then
    begin
     	V := Buffer;
  		Fl := V.dwFileFlags;
			Result := AnalyzeFlags(Fl);
    end;
	end;

begin
	StrPCopy(Name, FileName);
	BlockSize := GetFileVersionInfoSize(Name,Dummy);

	if BlockSize > 0 then begin
	  GetMem(PBlock,2*BlockSize);
	  try
			if GetFileVersionInfo(Name,Dummy,BlockSize,PBlock) then begin
		    V.CompanyName 		 := VerValue(PBlock,'CompanyName');
		    V.FileDescription  := VerValue(PBlock,'FileDescription');
		    V.FileVersion			 := VerValue(PBlock,'FileVersion');
		    V.InternalName		 := VerValue(PBlock,'InternalName');
		    V.LegalCopyright	 := VerValue(PBlock,'LegalCopyright');
	      V.OriginalFileName := VerValue(PBlock,'OriginalFileName');
		    V.ProductName			 := VerValue(PBlock,'ProductName');
		    V.ProductVersion	 := VerValue(PBlock,'ProductVersion');
		    V.Comments	       := VerValue(PBlock,'Comments');
        V.FileFlags        := VerFixedValue(PBlock);
  		end;
	  finally
  	  FreeMem(PBlock);
		end;
  end;
  Result := V ;
end;

end.
